@@ -7,11 +7,12 @@
 import { Disposable, Event, EventEmitter, FileDecoration, FileDecorationProvider, SourceControlHistoryItem, SourceControlHistoryItemChange, SourceControlHistoryOptions, SourceControlHistoryProvider, ThemeIcon, Uri, window, LogOutputChannel, SourceControlHistoryItemRef, l10n, SourceControlHistoryItemRefsChangeEvent } from 'vscode';
 import { Repository, Resource } from './repository';
 import { IDisposable, deltaHistoryItemRefs, dispose, filterEvent, getCommitShortHash } from './util';
-import { toGitUri } from './uri';
-import { Branch, LogOptions, Ref, RefType } from './api/git';
+import { toMultiFileDiffEditorUris } from './uri';
+import { AvatarQuery, AvatarQueryCommit, Branch, LogOptions, Ref, RefType } from './api/git';
 import { emojify, ensureEmojis } from './emoji';
 import { Commit } from './git';
 import { OperationKind, OperationResult } from './operation';
+import { ISourceControlHistoryItemDetailsProviderRegistry, provideSourceControlHistoryItemAvatar, provideSourceControlHistoryItemMessageLinks } from './historyItemDetailsProvider';
 
 function toSourceControlHistoryItemRef(repository: Repository, ref: Ref): SourceControlHistoryItemRef {
 	const rootUri = Uri.file(repository.root);
@@ -96,7 +97,11 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 
 	private disposables: Disposable[] = [];
 
-	constructor(protected readonly repository: Repository, private readonly logger: LogOutputChannel) {
+	constructor(
+		private historyItemDetailProviderRegistry: ISourceControlHistoryItemDetailsProviderRegistry,
+		private readonly repository: Repository,
+		private readonly logger: LogOutputChannel
+	) {
 		const onDidRunWriteOperation = filterEvent(repository.onDidRunOperation, e => !e.operation.readOnly);
 		this.disposables.push(onDidRunWriteOperation(this.onDidRunWriteOperation, this));
 
@@ -131,12 +136,27 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 					historyItemRefName = this.repository.HEAD.name;
 
 					// Remote
-					this._currentHistoryItemRemoteRef = this.repository.HEAD.upstream ? {
-						id: `refs/remotes/${this.repository.HEAD.upstream.remote}/${this.repository.HEAD.upstream.name}`,
-						name: `${this.repository.HEAD.upstream.remote}/${this.repository.HEAD.upstream.name}`,
-						revision: this.repository.HEAD.upstream.commit,
-						icon: new ThemeIcon('cloud')
-					} : undefined;
+					if (this.repository.HEAD.upstream) {
+						if (this.repository.HEAD.upstream.remote === '.') {
+							// Local branch
+							this._currentHistoryItemRemoteRef = {
+								id: `refs/heads/${this.repository.HEAD.upstream.name}`,
+								name: this.repository.HEAD.upstream.name,
+								revision: this.repository.HEAD.upstream.commit,
+								icon: new ThemeIcon('gi-branch')
+							};
+						} else {
+							// Remote branch
+							this._currentHistoryItemRemoteRef = {
+								id: `refs/remotes/${this.repository.HEAD.upstream.remote}/${this.repository.HEAD.upstream.name}`,
+								name: `${this.repository.HEAD.upstream.remote}/${this.repository.HEAD.upstream.name}`,
+								revision: this.repository.HEAD.upstream.commit,
+								icon: new ThemeIcon('cloud')
+							};
+						}
+					} else {
+						this._currentHistoryItemRemoteRef = undefined;
+					}
 
 					// Base
 					if (this._HEAD?.name !== this.repository.HEAD.name) {
@@ -262,24 +282,51 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 
 			const commits = await this.repository.log({ ...logOptions, silent: true });
 
+			// Avatars
+			const avatarQuery = {
+				commits: commits.map(c => ({
+					hash: c.hash,
+					authorName: c.authorName,
+					authorEmail: c.authorEmail
+				} satisfies AvatarQueryCommit)),
+				size: 20
+			} satisfies AvatarQuery;
+
+			const commitAvatars = await provideSourceControlHistoryItemAvatar(
+				this.historyItemDetailProviderRegistry, this.repository, avatarQuery);
+
 			await ensureEmojis();
 
-			return commits.map(commit => {
+			const historyItems: SourceControlHistoryItem[] = [];
+			for (const commit of commits) {
+				const message = emojify(commit.message);
+				const messageWithLinks = await provideSourceControlHistoryItemMessageLinks(
+					this.historyItemDetailProviderRegistry, this.repository, message) ?? message;
+
+				const newLineIndex = message.indexOf('\n');
+				const subject = newLineIndex !== -1
+					? `${message.substring(0, newLineIndex)}\u2026`
+					: message;
+
+				const avatarUrl = commitAvatars?.get(commit.hash);
 				const references = this._resolveHistoryItemRefs(commit);
 
-				return {
+				historyItems.push({
 					id: commit.hash,
 					parentIds: commit.parents,
-					message: emojify(commit.message),
+					subject,
+					message: messageWithLinks,
 					author: commit.authorName,
 					authorEmail: commit.authorEmail,
-					icon: new ThemeIcon('git-commit'),
+					authorIcon: avatarUrl ? Uri.parse(avatarUrl) : new ThemeIcon('account'),
 					displayId: getCommitShortHash(Uri.file(this.repository.root), commit.hash),
 					timestamp: commit.authorDate?.getTime(),
 					statistics: commit.shortStat ?? { files: 0, insertions: 0, deletions: 0 },
 					references: references.length !== 0 ? references : undefined
-				};
-			});
+				} satisfies SourceControlHistoryItem);
+			}
+
+			return historyItems;
 		} catch (err) {
 			this.logger.error(`[GitHistoryProvider][provideHistoryItems] Failed to get history items with options '${JSON.stringify(options)}': ${err}`);
 			return [];
@@ -301,10 +348,8 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 			// History item change
 			historyItemChanges.push({
 				uri: historyItemUri,
-				originalUri: toGitUri(change.originalUri, historyItemParentId),
-				modifiedUri: toGitUri(change.uri, historyItemId),
-				renameUri: change.renameUri,
-			});
+				...toMultiFileDiffEditorUris(change, historyItemParentId, historyItemId)
+			} satisfies SourceControlHistoryItemChange);
 
 			// History item change decoration
 			const letter = Resource.getStatusLetter(change.status);
@@ -369,6 +414,7 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 						id: ref.substring('HEAD -> '.length),
 						name: ref.substring('HEAD -> refs/heads/'.length),
 						revision: commit.hash,
+						category: l10n.t('branches'),
 						icon: new ThemeIcon('target')
 					});
 					break;
@@ -377,6 +423,7 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 						id: ref,
 						name: ref.substring('refs/heads/'.length),
 						revision: commit.hash,
+						category: l10n.t('branches'),
 						icon: new ThemeIcon('git-branch')
 					});
 					break;
@@ -385,6 +432,7 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 						id: ref,
 						name: ref.substring('refs/remotes/'.length),
 						revision: commit.hash,
+						category: l10n.t('remote branches'),
 						icon: new ThemeIcon('cloud')
 					});
 					break;
@@ -393,6 +441,7 @@ export class GitHistoryProvider implements SourceControlHistoryProvider, FileDec
 						id: ref.substring('tag: '.length),
 						name: ref.substring('tag: refs/tags/'.length),
 						revision: commit.hash,
+						category: l10n.t('tags'),
 						icon: new ThemeIcon('tag')
 					});
 					break;
