@@ -43,7 +43,7 @@ import { InstantiationService } from '../../../platform/instantiation/common/ins
 import { ServiceCollection } from '../../../platform/instantiation/common/serviceCollection.js';
 import { ILanguagePackService } from '../../../platform/languagePacks/common/languagePacks.js';
 import { NativeLanguagePackService } from '../../../platform/languagePacks/node/languagePacks.js';
-import { ConsoleLogger, ILoggerService, ILogService } from '../../../platform/log/common/log.js';
+import { ConsoleLogger, ILoggerService, ILogService, LoggerGroup } from '../../../platform/log/common/log.js';
 import { LoggerChannelClient } from '../../../platform/log/common/logIpc.js';
 import product from '../../../platform/product/common/product.js';
 import { IProductService } from '../../../platform/product/common/productService.js';
@@ -162,7 +162,6 @@ class SharedProcessMain extends Disposable implements IClientConnectionFilter {
 		instantiationService.invokeFunction(accessor => {
 			const logService = accessor.get(ILogService);
 			const telemetryService = accessor.get(ITelemetryService);
-			const userDataProfilesService = accessor.get(IUserDataProfilesService);
 
 			// Log info
 			logService.trace('sharedProcess configuration', JSON.stringify(this.configuration));
@@ -173,10 +172,6 @@ class SharedProcessMain extends Disposable implements IClientConnectionFilter {
 			// Error handler
 			this.registerErrorHandler(logService);
 
-			// Report Profiles Info
-			this.reportProfilesInfo(telemetryService, userDataProfilesService);
-			this._register(userDataProfilesService.onDidChangeProfiles(() => this.reportProfilesInfo(telemetryService, userDataProfilesService)));
-
 			// Report Client OS/DE Info
 			this.reportClientOSInfo(telemetryService, logService);
 		});
@@ -219,7 +214,8 @@ class SharedProcessMain extends Disposable implements IClientConnectionFilter {
 		services.set(ILoggerService, loggerService);
 
 		// Log
-		const logger = this._register(loggerService.createLogger('sharedprocess', { name: localize('sharedLog', "Shared") }));
+		const sharedLogGroup: LoggerGroup = { id: 'shared', name: localize('sharedLog', "Shared") };
+		const logger = this._register(loggerService.createLogger('sharedprocess', { name: localize('sharedLog', "Shared"), group: sharedLogGroup }));
 		const consoleLogger = this._register(new ConsoleLogger(logger.getLevel()));
 		const logService = this._register(new LogService(logger, [consoleLogger]));
 		services.set(ILogService, logService);
@@ -273,7 +269,8 @@ class SharedProcessMain extends Disposable implements IClientConnectionFilter {
 		]);
 
 		// Request
-		const requestService = new RequestService(configurationService, environmentService, logService);
+		const networkLogger = this._register(loggerService.createLogger(`network-shared`, { name: localize('networkk', "Network"), group: sharedLogGroup }));
+		const requestService = new RequestService(configurationService, environmentService, this._register(new LogService(networkLogger)));
 		services.set(IRequestService, requestService);
 
 		// Checksum
@@ -453,20 +450,6 @@ class SharedProcessMain extends Disposable implements IClientConnectionFilter {
 		});
 	}
 
-	private reportProfilesInfo(telemetryService: ITelemetryService, userDataProfilesService: IUserDataProfilesService): void {
-		type ProfilesInfoClassification = {
-			owner: 'sandy081';
-			comment: 'Report profiles information';
-			count: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'Number of profiles' };
-		};
-		type ProfilesInfoEvent = {
-			count: number;
-		};
-		telemetryService.publicLog2<ProfilesInfoEvent, ProfilesInfoClassification>('profilesInfo', {
-			count: userDataProfilesService.profiles.length
-		});
-	}
-
 	private async reportClientOSInfo(telemetryService: ITelemetryService, logService: ILogService): Promise<void> {
 		if (isLinux) {
 			const [releaseInfo, displayProtocol] = await Promise.all([
