@@ -277,9 +277,20 @@ export class VisibleLinesCollection<T extends IVisibleLine> {
 		return false;
 	}
 
-	public onFlushed(e: viewEvents.ViewFlushedEvent): boolean {
+	public onFlushed(e: viewEvents.ViewFlushedEvent, flushDom?: boolean): boolean {
+		// No need to clear the dom node because a full .innerHTML will occur in
+		// ViewLayerRenderer._render, however the the fallbakc mechanism in the
+		// GPU renderer may cause this to be necessary as the .innerHTML call
+		// may not happen depending on the new state, leaving stale DOM nodes
+		// around.
+		if (flushDom) {
+			const start = this._linesCollection.getStartLineNumber();
+			const end = this._linesCollection.getEndLineNumber();
+			for (let i = start; i <= end; i++) {
+				this._linesCollection.getLine(i).getDomNode()?.remove();
+			}
+		}
 		this._linesCollection.flush();
-		// No need to clear the dom node because a full .innerHTML will occur in ViewLayerRenderer._render
 		return true;
 	}
 
