@@ -7,10 +7,11 @@ import { ILanguageIdCodec, ITreeSitterTokenizationSupport, TreeSitterTokenizatio
 import { LineTokens } from '../tokens/lineTokens.js';
 import { StandardTokenType } from '../encodedTokenAttributes.js';
 import { TextModel } from './textModel.js';
-import { ITreeSitterParserService } from '../services/treeSitterParserService.js';
 import { IModelContentChangedEvent } from '../textModelEvents.js';
 import { AbstractTokens } from './tokens.js';
 import { IDisposable, MutableDisposable } from '../../../base/common/lifecycle.js';
+import { ITreeSitterTokenizationStoreService } from './treeSitterTokenStoreService.js';
+import { Range } from '../core/range.js';
 
 export class TreeSitterTokens extends AbstractTokens {
 	private _tokenizationSupport: ITreeSitterTokenizationSupport | null = null;
@@ -20,7 +21,7 @@ export class TreeSitterTokens extends AbstractTokens {
 	constructor(languageIdCodec: ILanguageIdCodec,
 		textModel: TextModel,
 		languageId: () => string,
-		@ITreeSitterParserService private readonly _treeSitterService: ITreeSitterParserService) {
+		@ITreeSitterTokenizationStoreService private readonly _tokenStore: ITreeSitterTokenizationStoreService) {
 		super(languageIdCodec, textModel, languageId);
 
 		this._initialize();
@@ -42,7 +43,7 @@ export class TreeSitterTokens extends AbstractTokens {
 	public getLineTokens(lineNumber: number): LineTokens {
 		const content = this._textModel.getLineContent(lineNumber);
 		if (this._tokenizationSupport) {
-			const rawTokens = this._tokenizationSupport.tokenizeEncoded(lineNumber, this._textModel);
+			const rawTokens = this._tokenStore.getTokens(this._textModel, lineNumber);
 			if (rawTokens) {
 				return new LineTokens(rawTokens, content, this._languageIdCodec);
 			}
@@ -77,16 +78,17 @@ export class TreeSitterTokens extends AbstractTokens {
 	}
 
 	public override forceTokenization(lineNumber: number): void {
-		// TODO @alexr00 implement
+		if (this._tokenizationSupport) {
+			this._tokenizationSupport.tokenizeEncoded(lineNumber, this._textModel);
+		}
 	}
 
 	public override hasAccurateTokensForLine(lineNumber: number): boolean {
-		// TODO @alexr00 update for background tokenization
-		return true;
+		return this._tokenStore.hasTokens(this._textModel, new Range(lineNumber, 1, lineNumber, this._textModel.getLineMaxColumn(lineNumber)));
 	}
 
 	public override isCheapToTokenize(lineNumber: number): boolean {
-		// TODO @alexr00 update for background tokenization
+		// TODO @alexr00 determine what makes it cheap to tokenize?
 		return true;
 	}
 
@@ -99,8 +101,6 @@ export class TreeSitterTokens extends AbstractTokens {
 		return null;
 	}
 	public override get hasTokens(): boolean {
-		// TODO @alexr00 once we have a token store, implement properly
-		const hasTree = this._treeSitterService.getParseResult(this._textModel) !== undefined;
-		return hasTree;
+		return this._tokenStore.hasTokens(this._textModel);
 	}
 }
