@@ -337,6 +337,8 @@ export class InlineCompletionItem {
 		);
 	}
 
+	static ID = 1;
+
 	private _didCallShow = false;
 
 	constructor(
@@ -362,7 +364,10 @@ export class InlineCompletionItem {
 		 * Used for event data to ensure referential equality.
 		*/
 		readonly source: InlineCompletionList,
+
+		readonly id = `InlineCompletion:${InlineCompletionItem.ID++}`,
 	) {
+		// TODO: these statements are no-ops
 		filterText = filterText.replace(/\r\n|\r/g, '\n');
 		insertText = filterText.replace(/\r\n|\r/g, '\n');
 	}
@@ -386,6 +391,23 @@ export class InlineCompletionItem {
 			this.additionalTextEdits,
 			this.sourceInlineCompletion,
 			this.source,
+			this.id,
+		);
+	}
+
+	public withRangeInsertTextAndFilterText(updatedRange: Range, updatedInsertText: string, updatedFilterText: string): InlineCompletionItem {
+		return new InlineCompletionItem(
+			updatedFilterText,
+			this.command,
+			this.shownCommand,
+			updatedRange,
+			updatedInsertText,
+			this.snippetInfo,
+			this.cursorShowRange,
+			this.additionalTextEdits,
+			this.sourceInlineCompletion,
+			this.source,
+			this.id,
 		);
 	}
 
