@@ -6,14 +6,16 @@
 import type { Parser } from '@vscode/tree-sitter-wasm';
 import { Event } from '../../../../base/common/event.js';
 import { ITextModel } from '../../../common/model.js';
-import { ITreeSitterParserService, ITreeSitterParseResult } from '../../../common/services/treeSitterParserService.js';
-import { Range } from '../../../common/core/range.js';
+import { ITreeSitterParserService, ITreeSitterParseResult, ITextModelTreeSitter, TreeUpdateEvent } from '../../../common/services/treeSitterParserService.js';
 
 export class TestTreeSitterParserService implements ITreeSitterParserService {
+	async getTextModelTreeSitter(model: ITextModel, parseImmediately?: boolean): Promise<ITextModelTreeSitter> {
+		throw new Error('Method not implemented.');
+	}
 	getTree(content: string, languageId: string): Promise<Parser.Tree | undefined> {
 		throw new Error('Method not implemented.');
 	}
-	onDidUpdateTree: Event<{ textModel: ITextModel; ranges: Range[] }> = Event.None;
+	onDidUpdateTree: Event<TreeUpdateEvent> = Event.None;
 	onDidAddLanguage: Event<{ id: string; language: Parser.Language }> = Event.None;
 	_serviceBrand: undefined;
 	getOrInitLanguage(languageId: string): Parser.Language | undefined {
