@@ -8,7 +8,6 @@ import { Promises, Queue } from '../../../base/common/async.js';
 import { VSBuffer } from '../../../base/common/buffer.js';
 import { CancellationToken } from '../../../base/common/cancellation.js';
 import { IStringDictionary } from '../../../base/common/collections.js';
-import { toErrorMessage } from '../../../base/common/errorMessage.js';
 import { CancellationError, getErrorMessage } from '../../../base/common/errors.js';
 import { Emitter } from '../../../base/common/event.js';
 import { hash } from '../../../base/common/hash.js';
@@ -18,7 +17,7 @@ import { Schemas } from '../../../base/common/network.js';
 import * as path from '../../../base/common/path.js';
 import { joinPath } from '../../../base/common/resources.js';
 import * as semver from '../../../base/common/semver/semver.js';
-import { isBoolean } from '../../../base/common/types.js';
+import { isBoolean, isDefined, isUndefined } from '../../../base/common/types.js';
 import { URI } from '../../../base/common/uri.js';
 import { generateUuid } from '../../../base/common/uuid.js';
 import * as pfs from '../../../base/node/pfs.js';
@@ -34,10 +33,11 @@ import {
 	EXTENSION_INSTALL_CLIENT_TARGET_PLATFORM_CONTEXT,
 	ExtensionSignatureVerificationCode,
 	computeSize,
+	IAllowedExtensionsService,
 } from '../common/extensionManagement.js';
 import { areSameExtensions, computeTargetPlatform, ExtensionKey, getGalleryExtensionId, groupByExtension } from '../common/extensionManagementUtil.js';
 import { IExtensionsProfileScannerService, IScannedProfileExtension } from '../common/extensionsProfileScannerService.js';
-import { IExtensionsScannerService, IScannedExtension, ScanOptions } from '../common/extensionsScannerService.js';
+import { IExtensionsScannerService, IScannedExtension, UserExtensionsScanOptions } from '../common/extensionsScannerService.js';
 import { ExtensionsDownloader } from './extensionDownloader.js';
 import { ExtensionsLifecycle } from './extensionLifecycle.js';
 import { fromExtractError, getManifest } from './extensionManagementUtil.js';
@@ -45,7 +45,7 @@ import { ExtensionsManifestCache } from './extensionsManifestCache.js';
 import { DidChangeProfileExtensionsEvent, ExtensionsWatcher } from './extensionsWatcher.js';
 import { ExtensionType, IExtension, IExtensionManifest, TargetPlatform } from '../../extensions/common/extensions.js';
 import { isEngineValid } from '../../extensions/common/extensionValidator.js';
-import { FileChangesEvent, FileChangeType, FileOperationResult, IFileService, toFileOperationResult } from '../../files/common/files.js';
+import { FileChangesEvent, FileChangeType, FileOperationResult, IFileService, IFileStat, toFileOperationResult } from '../../files/common/files.js';
 import { IInstantiationService, refineServiceDecorator } from '../../instantiation/common/instantiation.js';
 import { ILogService } from '../../log/common/log.js';
 import { IProductService } from '../../product/common/productService.js';
@@ -60,7 +60,7 @@ export interface INativeServerExtensionManagementService extends IExtensionManag
 	readonly _serviceBrand: undefined;
 	scanAllUserInstalledExtensions(): Promise<ILocalExtension[]>;
 	scanInstalledExtensionAtLocation(location: URI): Promise<ILocalExtension | null>;
-	markAsUninstalled(...extensions: IExtension[]): Promise<void>;
+	deleteExtensions(...extensions: IExtension[]): Promise<void>;
 }
 
 type ExtractExtensionResult = { readonly local: ILocalExtension; readonly verificationStatus?: ExtensionSignatureVerificationCode };
@@ -87,10 +87,11 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 		@IFileService private readonly fileService: IFileService,
 		@IConfigurationService private readonly configurationService: IConfigurationService,
 		@IProductService productService: IProductService,
+		@IAllowedExtensionsService allowedExtensionsService: IAllowedExtensionsService,
 		@IUriIdentityService uriIdentityService: IUriIdentityService,
 		@IUserDataProfilesService userDataProfilesService: IUserDataProfilesService
 	) {
-		super(galleryService, telemetryService, uriIdentityService, logService, productService, userDataProfilesService);
+		super(galleryService, telemetryService, uriIdentityService, logService, productService, allowedExtensionsService, userDataProfilesService);
 		const extensionLifecycle = this._register(instantiationService.createInstance(ExtensionsLifecycle));
 		this.extensionsScanner = this._register(instantiationService.createInstance(ExtensionsScanner, extension => extensionLifecycle.postUninstall(extension)));
 		this.manifestCache = this._register(new ExtensionsManifestCache(userDataProfilesService, fileService, uriIdentityService, this, this.logService));
@@ -131,7 +132,7 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 	}
 
 	scanAllUserInstalledExtensions(): Promise<ILocalExtension[]> {
-		return this.extensionsScanner.scanAllUserExtensions(false);
+		return this.extensionsScanner.scanAllUserExtensions();
 	}
 
 	scanInstalledExtensionAtLocation(location: URI): Promise<ILocalExtension | null> {
@@ -150,6 +151,11 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 				throw new Error(nls.localize('incompatible', "Unable to install extension '{0}' as it is not compatible with VS Code '{1}'.", extensionId, this.productService.version));
 			}
 
+			const allowedToInstall = this.allowedExtensionsService.isAllowed({ id: extensionId, version: manifest.version, publisherDisplayName: undefined });
+			if (allowedToInstall !== true) {
+				throw new Error(nls.localize('notAllowed', "This extension cannot be installed because {0}", allowedToInstall.value));
+			}
+
 			const results = await this.installExtensions([{ manifest, extension: location, options }]);
 			const result = results.find(({ identifier }) => areSameExtensions(identifier, { id: extensionId }));
 			if (result?.local) {
@@ -208,27 +214,6 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 		return local;
 	}
 
-	async reinstallFromGallery(extension: ILocalExtension): Promise<ILocalExtension> {
-		this.logService.trace('ExtensionManagementService#reinstallFromGallery', extension.identifier.id);
-		if (!this.galleryService.isEnabled()) {
-			throw new Error(nls.localize('MarketPlaceDisabled', "Marketplace is not enabled"));
-		}
-
-		const targetPlatform = await this.getTargetPlatform();
-		const [galleryExtension] = await this.galleryService.getExtensions([{ ...extension.identifier, preRelease: extension.preRelease }], { targetPlatform, compatible: true }, CancellationToken.None);
-		if (!galleryExtension) {
-			throw new Error(nls.localize('Not a Marketplace extension', "Only Marketplace Extensions can be reinstalled"));
-		}
-
-		await this.extensionsScanner.setUninstalled(extension);
-		try {
-			await this.extensionsScanner.removeUninstalledExtension(extension);
-		} catch (e) {
-			throw new Error(nls.localize('removeError', "Error while removing the extension: {0}. Please Quit and Start VS Code before trying again.", toErrorMessage(e)));
-		}
-		return this.installFromGallery(galleryExtension);
-	}
-
 	protected copyExtension(extension: ILocalExtension, fromProfileLocation: URI, toProfileLocation: URI, metadata: Partial<Metadata>): Promise<ILocalExtension> {
 		return this.extensionsScanner.copyExtension(extension, fromProfileLocation, toProfileLocation, metadata);
 	}
@@ -237,8 +222,8 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 		return this.extensionsScanner.copyExtensions(fromProfileLocation, toProfileLocation, { version: this.productService.version, date: this.productService.date });
 	}
 
-	markAsUninstalled(...extensions: IExtension[]): Promise<void> {
-		return this.extensionsScanner.setUninstalled(...extensions);
+	deleteExtensions(...extensions: IExtension[]): Promise<void> {
+		return this.extensionsScanner.setExtensionsForRemoval(...extensions);
 	}
 
 	async cleanUp(): Promise<void> {
@@ -313,25 +298,18 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 			const local = await this.extensionsScanner.extractUserExtension(
 				extensionKey,
 				location.fsPath,
-				{
-					id: gallery.identifier.uuid,
-					publisherId: gallery.publisherId,
-					publisherDisplayName: gallery.publisherDisplayName,
-					targetPlatform: gallery.properties.targetPlatform,
-					isApplicationScoped: options.isApplicationScoped,
-					isMachineScoped: options.isMachineScoped,
-					isBuiltin: options.isBuiltin,
-					isPreReleaseVersion: gallery.properties.isPreReleaseVersion,
-					hasPreReleaseVersion: gallery.properties.isPreReleaseVersion,
-					installedTimestamp: Date.now(),
-					pinned: options.installGivenVersion ? true : !!options.pinned,
-					preRelease: isBoolean(options.preRelease)
-						? options.preRelease
-						: options.installPreReleaseVersion || gallery.properties.isPreReleaseVersion,
-					source: 'gallery',
-				},
 				false,
 				token);
+
+			if (verificationStatus !== ExtensionSignatureVerificationCode.Success && this.environmentService.isBuilt) {
+				try {
+					await this.extensionsDownloader.delete(location);
+				} catch (e) {
+					/* Ignore */
+					this.logService.warn(`Error while deleting the downloaded file`, location.toString(), getErrorMessage(e));
+				}
+			}
+
 			return { local, verificationStatus };
 		} catch (error) {
 			try {
@@ -351,9 +329,12 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 		}
 		const { location, verificationStatus } = await this.extensionsDownloader.download(extension, operation, verifySignature, clientTargetPlatform);
 
-		if (verificationStatus !== ExtensionSignatureVerificationCode.Success && verifySignature && this.environmentService.isBuilt && !isLinux) {
-			if (!extension.isSigned) {
-				throw new ExtensionManagementError(nls.localize('not signed', "Extension is not signed."), ExtensionManagementErrorCode.PackageNotSigned);
+		if (verificationStatus !== ExtensionSignatureVerificationCode.Success && verificationStatus !== ExtensionSignatureVerificationCode.NotSigned && verifySignature && this.environmentService.isBuilt && !isLinux) {
+			try {
+				await this.extensionsDownloader.delete(location);
+			} catch (e) {
+				/* Ignore */
+				this.logService.warn(`Error while deleting the downloaded file`, location.toString(), getErrorMessage(e));
 			}
 
 			if (!verificationStatus) {
@@ -384,15 +365,7 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 		const local = await this.extensionsScanner.extractUserExtension(
 			extensionKey,
 			path.resolve(location.fsPath),
-			{
-				isApplicationScoped: options.isApplicationScoped,
-				isMachineScoped: options.isMachineScoped,
-				isBuiltin: options.isBuiltin,
-				installedTimestamp: Date.now(),
-				pinned: options.installGivenVersion ? true : !!options.pinned,
-				source: 'vsix',
-			},
-			options.keepExisting ?? true,
+			isBoolean(options.keepExisting) ? !options.keepExisting : true,
 			token);
 		return { local };
 	}
@@ -482,8 +455,20 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 				continue;
 			}
 
-			// Check if this is a directory
-			if (!(await this.fileService.stat(resource)).isDirectory) {
+			// Ignore changes to the deleted folder
+			if (this.uriIdentityService.extUri.basename(resource).endsWith(DELETED_FOLDER_POSTFIX)) {
+				continue;
+			}
+
+			try {
+				// Check if this is a directory
+				if (!(await this.fileService.stat(resource)).isDirectory) {
+					continue;
+				}
+			} catch (error) {
+				if (toFileOperationResult(error) !== FileOperationResult.FILE_NOT_FOUND) {
+					this.logService.error(error);
+				}
 				continue;
 			}
 
@@ -504,23 +489,10 @@ export class ExtensionManagementService extends AbstractExtensionManagementServi
 
 	private async addExtensionsToProfile(extensions: [ILocalExtension, Metadata | undefined][], profileLocation: URI): Promise<void> {
 		const localExtensions = extensions.map(e => e[0]);
-		await this.setInstalled(localExtensions);
+		await this.extensionsScanner.unsetExtensionsForRemoval(...localExtensions.map(extension => ExtensionKey.create(extension)));
 		await this.extensionsProfileScannerService.addExtensionsToProfile(extensions, profileLocation);
 		this._onDidInstallExtensions.fire(localExtensions.map(local => ({ local, identifier: local.identifier, operation: InstallOperation.None, profileLocation })));
 	}
-
-	private async setInstalled(extensions: ILocalExtension[]): Promise<void> {
-		const uninstalled = await this.extensionsScanner.getUninstalledExtensions();
-		for (const extension of extensions) {
-			const extensionKey = ExtensionKey.create(extension);
-			if (!uninstalled[extensionKey.toString()]) {
-				continue;
-			}
-			this.logService.trace('Removing the extension from uninstalled list:', extensionKey.id);
-			await this.extensionsScanner.setInstalled(extensionKey);
-			this.logService.info('Removed the extension from uninstalled list:', extensionKey.id);
-		}
-	}
 }
 
 type UpdateMetadataErrorClassification = {
@@ -538,8 +510,8 @@ type UpdateMetadataErrorEvent = {
 
 export class ExtensionsScanner extends Disposable {
 
-	private readonly uninstalledResource: URI;
-	private readonly uninstalledFileLimiter: Queue<any>;
+	private readonly obsoletedResource: URI;
+	private readonly obsoleteFileLimiter: Queue<any>;
 
 	private readonly _onExtract = this._register(new Emitter<URI>());
 	readonly onExtract = this._onExtract.event;
@@ -557,24 +529,25 @@ export class ExtensionsScanner extends Disposable {
 		@ILogService private readonly logService: ILogService,
 	) {
 		super();
-		this.uninstalledResource = joinPath(this.extensionsScannerService.userExtensionsLocation, '.obsolete');
-		this.uninstalledFileLimiter = new Queue();
+		this.obsoletedResource = joinPath(this.extensionsScannerService.userExtensionsLocation, '.obsolete');
+		this.obsoleteFileLimiter = new Queue();
 	}
 
 	async cleanUp(): Promise<void> {
 		await this.removeTemporarilyDeletedFolders();
-		await this.removeUninstalledExtensions();
-		await this.initializeMetadata();
+		await this.deleteExtensionsMarkedForRemoval();
+		//TODO: Remove this initiialization after coupe of releases
+		await this.initializeExtensionSize();
 	}
 
 	async scanExtensions(type: ExtensionType | null, profileLocation: URI, productVersion: IProductVersion): Promise<ILocalExtension[]> {
 		try {
-			const userScanOptions: ScanOptions = { includeInvalid: true, profileLocation, productVersion };
+			const userScanOptions: UserExtensionsScanOptions = { includeInvalid: true, profileLocation, productVersion };
 			let scannedExtensions: IScannedExtension[] = [];
 			if (type === null || type === ExtensionType.System) {
 				let scanAllExtensionsPromise = this.scanAllExtensionPromise.get(profileLocation);
 				if (!scanAllExtensionsPromise) {
-					scanAllExtensionsPromise = this.extensionsScannerService.scanAllExtensions({ includeInvalid: true, useCache: true }, userScanOptions, false)
+					scanAllExtensionsPromise = this.extensionsScannerService.scanAllExtensions({}, userScanOptions)
 						.finally(() => this.scanAllExtensionPromise.delete(profileLocation));
 					this.scanAllExtensionPromise.set(profileLocation, scanAllExtensionsPromise);
 				}
@@ -595,9 +568,9 @@ export class ExtensionsScanner extends Disposable {
 		}
 	}
 
-	async scanAllUserExtensions(excludeOutdated: boolean): Promise<ILocalExtension[]> {
+	async scanAllUserExtensions(): Promise<ILocalExtension[]> {
 		try {
-			const scannedExtensions = await this.extensionsScannerService.scanUserExtensions({ includeAllVersions: !excludeOutdated, includeInvalid: true });
+			const scannedExtensions = await this.extensionsScannerService.scanAllUserExtensions();
 			return await Promise.all(scannedExtensions.map(extension => this.toLocalExtension(extension)));
 		} catch (error) {
 			throw toExtensionManagementError(error, ExtensionManagementErrorCode.Scanning);
@@ -616,7 +589,7 @@ export class ExtensionsScanner extends Disposable {
 		return null;
 	}
 
-	async extractUserExtension(extensionKey: ExtensionKey, zipPath: string, metadata: Metadata, removeIfExists: boolean, token: CancellationToken): Promise<ILocalExtension> {
+	async extractUserExtension(extensionKey: ExtensionKey, zipPath: string, removeIfExists: boolean, token: CancellationToken): Promise<ILocalExtension> {
 		const folderName = extensionKey.toString();
 		const tempLocation = URI.file(path.join(this.extensionsScannerService.userExtensionsLocation.fsPath, `.${generateUuid()}`));
 		const extensionLocation = URI.file(path.join(this.extensionsScannerService.userExtensionsLocation.fsPath, folderName));
@@ -651,6 +624,7 @@ export class ExtensionsScanner extends Disposable {
 				throw fromExtractError(e);
 			}
 
+			const metadata: Metadata = { installedTimestamp: Date.now() };
 			try {
 				metadata.size = await computeSize(tempLocation, this.fileService);
 			} catch (error) {
@@ -694,13 +668,9 @@ export class ExtensionsScanner extends Disposable {
 		return this.scanLocalExtension(extensionLocation, ExtensionType.User);
 	}
 
-	async scanMetadata(local: ILocalExtension, profileLocation?: URI): Promise<Metadata | undefined> {
-		if (profileLocation) {
-			const extension = await this.getScannedExtension(local, profileLocation);
-			return extension?.metadata;
-		} else {
-			return this.extensionsScannerService.scanMetadata(local.location);
-		}
+	async scanMetadata(local: ILocalExtension, profileLocation: URI): Promise<Metadata | undefined> {
+		const extension = await this.getScannedExtension(local, profileLocation);
+		return extension?.metadata;
 	}
 
 	private async getScannedExtension(local: ILocalExtension, profileLocation: URI): Promise<IScannedProfileExtension | undefined> {
@@ -722,42 +692,40 @@ export class ExtensionsScanner extends Disposable {
 		return this.scanLocalExtension(local.location, local.type, profileLocation);
 	}
 
-	async getUninstalledExtensions(): Promise<IStringDictionary<boolean>> {
-		try {
-			return await this.withUninstalledExtensions();
-		} catch (error) {
-			throw toExtensionManagementError(error, ExtensionManagementErrorCode.ReadUninstalled);
-		}
-	}
-
-	async setUninstalled(...extensions: IExtension[]): Promise<void> {
+	async setExtensionsForRemoval(...extensions: IExtension[]): Promise<void> {
 		const extensionKeys: ExtensionKey[] = extensions.map(e => ExtensionKey.create(e));
-		await this.withUninstalledExtensions(uninstalled =>
+		await this.withRemovedExtensions(removedExtensions =>
 			extensionKeys.forEach(extensionKey => {
-				uninstalled[extensionKey.toString()] = true;
-				this.logService.info('Marked extension as uninstalled', extensionKey.toString());
+				removedExtensions[extensionKey.toString()] = true;
+				this.logService.info('Marked extension as removed', extensionKey.toString());
 			}));
 	}
 
-	async setInstalled(extensionKey: ExtensionKey): Promise<void> {
+	async unsetExtensionsForRemoval(...extensionKeys: ExtensionKey[]): Promise<boolean[]> {
 		try {
-			await this.withUninstalledExtensions(uninstalled => delete uninstalled[extensionKey.toString()]);
+			const results: boolean[] = [];
+			await this.withRemovedExtensions(removedExtensions =>
+				extensionKeys.forEach(extensionKey => {
+					if (removedExtensions[extensionKey.toString()]) {
+						results.push(true);
+						delete removedExtensions[extensionKey.toString()];
+					} else {
+						results.push(false);
+					}
+				}));
+			return results;
 		} catch (error) {
-			throw toExtensionManagementError(error, ExtensionManagementErrorCode.UnsetUninstalled);
+			throw toExtensionManagementError(error, ExtensionManagementErrorCode.UnsetRemoved);
 		}
 	}
 
-	async removeExtension(extension: ILocalExtension | IScannedExtension, type: string): Promise<void> {
+	async deleteExtension(extension: ILocalExtension | IScannedExtension, type: string): Promise<void> {
 		if (this.uriIdentityService.extUri.isEqualOrParent(extension.location, this.extensionsScannerService.userExtensionsLocation)) {
-			return this.deleteExtensionFromLocation(extension.identifier.id, extension.location, type);
+			await this.deleteExtensionFromLocation(extension.identifier.id, extension.location, type);
+			await this.unsetExtensionsForRemoval(ExtensionKey.create(extension));
 		}
 	}
 
-	async removeUninstalledExtension(extension: ILocalExtension | IScannedExtension): Promise<void> {
-		await this.removeExtension(extension, 'uninstalled');
-		await this.withUninstalledExtensions(uninstalled => delete uninstalled[ExtensionKey.create(extension).toString()]);
-	}
-
 	async copyExtension(extension: ILocalExtension, fromProfileLocation: URI, toProfileLocation: URI, metadata: Partial<Metadata>): Promise<ILocalExtension> {
 		const source = await this.getScannedExtension(extension, fromProfileLocation);
 		const target = await this.getScannedExtension(extension, toProfileLocation);
@@ -768,7 +736,7 @@ export class ExtensionsScanner extends Disposable {
 				await this.extensionsProfileScannerService.updateMetadata([[extension, { ...target.metadata, ...metadata }]], toProfileLocation);
 			} else {
 				const targetExtension = await this.scanLocalExtension(target.location, extension.type, toProfileLocation);
-				await this.extensionsProfileScannerService.removeExtensionFromProfile(targetExtension, toProfileLocation);
+				await this.extensionsProfileScannerService.removeExtensionsFromProfile([targetExtension.identifier], toProfileLocation);
 				await this.extensionsProfileScannerService.addExtensionsToProfile([[extension, { ...target.metadata, ...metadata }]], toProfileLocation);
 			}
 		} else {
@@ -794,11 +762,11 @@ export class ExtensionsScanner extends Disposable {
 		this.logService.info(`Deleted ${type} extension from disk`, id, location.fsPath);
 	}
 
-	private withUninstalledExtensions(updateFn?: (uninstalled: IStringDictionary<boolean>) => void): Promise<IStringDictionary<boolean>> {
-		return this.uninstalledFileLimiter.queue(async () => {
+	private withRemovedExtensions(updateFn?: (removed: IStringDictionary<boolean>) => void): Promise<IStringDictionary<boolean>> {
+		return this.obsoleteFileLimiter.queue(async () => {
 			let raw: string | undefined;
 			try {
-				const content = await this.fileService.readFile(this.uninstalledResource, 'utf8');
+				const content = await this.fileService.readFile(this.obsoletedResource, 'utf8');
 				raw = content.value.toString();
 			} catch (error) {
 				if (toFileOperationResult(error) !== FileOperationResult.FILE_NOT_FOUND) {
@@ -806,23 +774,29 @@ export class ExtensionsScanner extends Disposable {
 				}
 			}
 
-			let uninstalled = {};
+			let removed = {};
 			if (raw) {
 				try {
-					uninstalled = JSON.parse(raw);
+					removed = JSON.parse(raw);
 				} catch (e) { /* ignore */ }
 			}
 
 			if (updateFn) {
-				updateFn(uninstalled);
-				if (Object.keys(uninstalled).length) {
-					await this.fileService.writeFile(this.uninstalledResource, VSBuffer.fromString(JSON.stringify(uninstalled)));
+				updateFn(removed);
+				if (Object.keys(removed).length) {
+					await this.fileService.writeFile(this.obsoletedResource, VSBuffer.fromString(JSON.stringify(removed)));
 				} else {
-					await this.fileService.del(this.uninstalledResource);
+					try {
+						await this.fileService.del(this.obsoletedResource);
+					} catch (error) {
+						if (toFileOperationResult(error) !== FileOperationResult.FILE_NOT_FOUND) {
+							throw error;
+						}
+					}
 				}
 			}
 
-			return uninstalled;
+			return removed;
 		});
 	}
 
@@ -855,10 +829,14 @@ export class ExtensionsScanner extends Disposable {
 	}
 
 	private async toLocalExtension(extension: IScannedExtension): Promise<ILocalExtension> {
-		const stat = await this.fileService.resolve(extension.location);
+		let stat: IFileStat | undefined;
+		try {
+			stat = await this.fileService.resolve(extension.location);
+		} catch (error) {/* ignore */ }
+
 		let readmeUrl: URI | undefined;
 		let changelogUrl: URI | undefined;
-		if (stat.children) {
+		if (stat?.children) {
 			readmeUrl = stat.children.find(({ name }) => /^readme(\.txt|\.md|)$/i.test(name))?.resource;
 			changelogUrl = stat.children.find(({ name }) => /^changelog(\.txt|\.md|)$/i.test(name))?.resource;
 		}
@@ -879,7 +857,7 @@ export class ExtensionsScanner extends Disposable {
 			isMachineScoped: !!extension.metadata?.isMachineScoped,
 			isPreReleaseVersion: !!extension.metadata?.isPreReleaseVersion,
 			hasPreReleaseVersion: !!extension.metadata?.hasPreReleaseVersion,
-			preRelease: !!extension.metadata?.preRelease,
+			preRelease: extension.preRelease,
 			installedTimestamp: extension.metadata?.installedTimestamp,
 			updated: !!extension.metadata?.updated,
 			pinned: !!extension.metadata?.pinned,
@@ -889,30 +867,36 @@ export class ExtensionsScanner extends Disposable {
 		};
 	}
 
-	private async initializeMetadata(): Promise<void> {
-		const extensions = await this.extensionsScannerService.scanUserExtensions({ includeInvalid: true });
+	private async initializeExtensionSize(): Promise<void> {
+		const extensions = await this.extensionsScannerService.scanAllUserExtensions();
 		await Promise.all(extensions.map(async extension => {
 			// set size if not set before
-			if (!extension.metadata?.size && extension.metadata?.source !== 'resource') {
+			if (isDefined(extension.metadata?.installedTimestamp) && isUndefined(extension.metadata?.size)) {
 				const size = await computeSize(extension.location, this.fileService);
 				await this.extensionsScannerService.updateMetadata(extension.location, { size });
 			}
 		}));
 	}
 
-	private async removeUninstalledExtensions(): Promise<void> {
-		const uninstalled = await this.getUninstalledExtensions();
-		if (Object.keys(uninstalled).length === 0) {
-			this.logService.debug(`No uninstalled extensions found.`);
+	private async deleteExtensionsMarkedForRemoval(): Promise<void> {
+		let removed: IStringDictionary<boolean>;
+		try {
+			removed = await this.withRemovedExtensions();
+		} catch (error) {
+			throw toExtensionManagementError(error, ExtensionManagementErrorCode.ReadRemoved);
+		}
+
+		if (Object.keys(removed).length === 0) {
+			this.logService.debug(`No extensions are marked as removed.`);
 			return;
 		}
 
-		this.logService.debug(`Removing uninstalled extensions:`, Object.keys(uninstalled));
+		this.logService.debug(`Deleting extensions marked as removed:`, Object.keys(removed));
 
-		const extensions = await this.extensionsScannerService.scanUserExtensions({ includeAllVersions: true, includeUninstalled: true, includeInvalid: true }); // All user extensions
+		const extensions = await this.scanAllUserExtensions();
 		const installed: Set<string> = new Set<string>();
 		for (const e of extensions) {
-			if (!uninstalled[ExtensionKey.create(e).toString()]) {
+			if (!removed[ExtensionKey.create(e).toString()]) {
 				installed.add(e.identifier.id.toLowerCase());
 			}
 		}
@@ -923,15 +907,15 @@ export class ExtensionsScanner extends Disposable {
 			await Promises.settled(byExtension.map(async e => {
 				const latest = e.sort((a, b) => semver.rcompare(a.manifest.version, b.manifest.version))[0];
 				if (!installed.has(latest.identifier.id.toLowerCase())) {
-					await this.beforeRemovingExtension(await this.toLocalExtension(latest));
+					await this.beforeRemovingExtension(latest);
 				}
 			}));
 		} catch (error) {
 			this.logService.error(error);
 		}
 
-		const toRemove = extensions.filter(e => e.metadata /* Installed by System */ && uninstalled[ExtensionKey.create(e).toString()]);
-		await Promise.allSettled(toRemove.map(e => this.removeUninstalledExtension(e)));
+		const toRemove = extensions.filter(e => e.installedTimestamp /* Installed by System */ && removed[ExtensionKey.create(e).toString()]);
+		await Promise.allSettled(toRemove.map(e => this.deleteExtension(e, 'marked for removal')));
 	}
 
 	private async removeTemporarilyDeletedFolders(): Promise<void> {
@@ -1023,7 +1007,7 @@ class InstallExtensionInProfileTask extends AbstractExtensionTask<ILocalExtensio
 			if (existingExtension) {
 				if (this.extensionKey.equals(new ExtensionKey(existingExtension.identifier, existingExtension.manifest.version))) {
 					try {
-						await this.extensionsScanner.removeExtension(existingExtension, 'existing');
+						await this.extensionsScanner.deleteExtension(existingExtension, 'existing');
 					} catch (e) {
 						throw new Error(nls.localize('restartCode', "Please restart VS Code before reinstalling {0}.", this.manifest.displayName || this.manifest.name));
 					}
@@ -1031,10 +1015,10 @@ class InstallExtensionInProfileTask extends AbstractExtensionTask<ILocalExtensio
 			}
 			// Remove the extension with same version if it is already uninstalled.
 			// Installing a VSIX extension shall replace the existing extension always.
-			const existingWithSameVersion = await this.unsetIfUninstalled(this.extensionKey);
+			const existingWithSameVersion = await this.unsetIfRemoved(this.extensionKey);
 			if (existingWithSameVersion) {
 				try {
-					await this.extensionsScanner.removeExtension(existingWithSameVersion, 'existing');
+					await this.extensionsScanner.deleteExtension(existingWithSameVersion, 'existing');
 				} catch (e) {
 					throw new Error(nls.localize('restartCode', "Please restart VS Code before reinstalling {0}.", this.manifest.displayName || this.manifest.name));
 				}
@@ -1060,7 +1044,7 @@ class InstallExtensionInProfileTask extends AbstractExtensionTask<ILocalExtensio
 			}
 
 			// Unset if the extension is uninstalled and return the unset extension.
-			local = await this.unsetIfUninstalled(this.extensionKey);
+			local = await this.unsetIfRemoved(this.extensionKey);
 		}
 
 		if (token.isCancellationRequested) {
@@ -1103,19 +1087,15 @@ class InstallExtensionInProfileTask extends AbstractExtensionTask<ILocalExtensio
 		return result;
 	}
 
-	private async unsetIfUninstalled(extensionKey: ExtensionKey): Promise<ILocalExtension | undefined> {
-		const uninstalled = await this.extensionsScanner.getUninstalledExtensions();
-		if (!uninstalled[extensionKey.toString()]) {
-			return undefined;
+	private async unsetIfRemoved(extensionKey: ExtensionKey): Promise<ILocalExtension | undefined> {
+		// If the same version of extension is marked as removed, remove it from there and return the local.
+		const [removed] = await this.extensionsScanner.unsetExtensionsForRemoval(extensionKey);
+		if (removed) {
+			this.logService.info('Removed the extension from removed list:', extensionKey.id);
+			const userExtensions = await this.extensionsScanner.scanAllUserExtensions();
+			return userExtensions.find(i => ExtensionKey.create(i).equals(extensionKey));
 		}
-
-		this.logService.trace('Removing the extension from uninstalled list:', extensionKey.id);
-		// If the same version of extension is marked as uninstalled, remove it from there and return the local.
-		await this.extensionsScanner.setInstalled(extensionKey);
-		this.logService.info('Removed the extension from uninstalled list:', extensionKey.id);
-
-		const userExtensions = await this.extensionsScanner.scanAllUserExtensions(true);
-		return userExtensions.find(i => ExtensionKey.create(i).equals(extensionKey));
+		return undefined;
 	}
 
 	private async updateMetadata(extension: ILocalExtension, token: CancellationToken): Promise<void> {
@@ -1151,8 +1131,8 @@ class UninstallExtensionInProfileTask extends AbstractExtensionTask<void> implem
 		super();
 	}
 
-	protected async doRun(token: CancellationToken): Promise<void> {
-		await this.extensionsProfileScannerService.removeExtensionFromProfile(this.extension, this.options.profileLocation);
+	protected doRun(token: CancellationToken): Promise<void> {
+		return this.extensionsProfileScannerService.removeExtensionsFromProfile([this.extension.identifier], this.options.profileLocation);
 	}
 
 }
