@@ -19,14 +19,15 @@ import { ChatAgentLocation } from '../../common/chatAgents.js';
 import { ChatContextKeys } from '../../common/chatContextKeys.js';
 import { hasAppliedChatEditsContextKey, hasUndecidedChatEditingResourceContextKey, IChatEditingService, WorkingSetEntryState } from '../../common/chatEditingService.js';
 import { ChatViewId, EditsViewId, IChatWidgetService } from '../chat.js';
+import { ctxIsGlobalEditingSession } from '../chatEditorController.js';
 import { ChatEditorInput } from '../chatEditorInput.js';
 import { ChatViewPane } from '../chatViewPane.js';
 import { CHAT_CATEGORY } from './chatActions.js';
 import { clearChatEditor } from './chatClear.js';
 
 export const ACTION_ID_NEW_CHAT = `workbench.action.chat.newChat`;
-
 export const ACTION_ID_NEW_EDIT_SESSION = `workbench.action.chat.newEditSession`;
+export const ChatDoneActionId = 'workbench.action.chat.done';
 
 export function registerNewChatActions() {
 	registerAction2(class NewChatEditorAction extends Action2 {
@@ -210,7 +211,7 @@ export function registerNewChatActions() {
 	registerAction2(class GlobalEditsDoneAction extends Action2 {
 		constructor() {
 			super({
-				id: 'workbench.action.chat.done',
+				id: ChatDoneActionId,
 				title: localize2('chat.done.label', "Done"),
 				category: CHAT_CATEGORY,
 				precondition: ContextKeyExpr.and(ChatContextKeys.enabled, ChatContextKeys.editingParticipantRegistered),
@@ -315,7 +316,6 @@ export function registerNewChatActions() {
 				title: localize2('chat.openEdits.label', "Open {0}", 'Copilot Edits'),
 				category: CHAT_CATEGORY,
 				icon: Codicon.goToEditingSession,
-				precondition: ContextKeyExpr.and(ChatContextKeys.enabled, ChatContextKeys.editingParticipantRegistered),
 				f1: true,
 				menu: [{
 					id: MenuId.ViewTitle,
@@ -333,6 +333,7 @@ export function registerNewChatActions() {
 					order: 2
 				}, {
 					id: MenuId.ChatEditingEditorContent,
+					when: ctxIsGlobalEditingSession,
 					group: 'navigate',
 					order: 4,
 				}],
