@@ -8,6 +8,7 @@ import { KeyCode, KeyMod } from '../../../../../base/common/keyCodes.js';
 import { URI } from '../../../../../base/common/uri.js';
 import { localize, localize2 } from '../../../../../nls.js';
 import { Action2, MenuId, MenuRegistry, registerAction2 } from '../../../../../platform/actions/common/actions.js';
+import { ICommandService } from '../../../../../platform/commands/common/commands.js';
 import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
 import { IDialogService } from '../../../../../platform/dialogs/common/dialogs.js';
 import { ServicesAccessor } from '../../../../../platform/instantiation/common/instantiation.js';
@@ -15,12 +16,14 @@ import { KeybindingWeight } from '../../../../../platform/keybinding/common/keyb
 import { IViewsService } from '../../../../services/views/common/viewsService.js';
 import { ChatAgentLocation, IChatAgentService } from '../../common/chatAgents.js';
 import { ChatContextKeys } from '../../common/chatContextKeys.js';
-import { applyingChatEditsContextKey, IChatEditingService } from '../../common/chatEditingService.js';
+import { applyingChatEditsContextKey, IChatEditingService, WorkingSetEntryState } from '../../common/chatEditingService.js';
 import { chatAgentLeader, extractAgentAndCommand } from '../../common/chatParserTypes.js';
 import { IChatService } from '../../common/chatService.js';
 import { EditsViewId, IChatWidget, IChatWidgetService } from '../chat.js';
+import { discardAllEditsWithConfirmation } from '../chatEditing/chatEditingActions.js';
 import { ChatViewPane } from '../chatViewPane.js';
 import { CHAT_CATEGORY } from './chatActions.js';
+import { ChatDoneActionId } from './chatClearActions.js';
 
 export interface IVoiceChatExecuteActionContext {
 	readonly disableTimeout?: boolean;
@@ -46,13 +49,21 @@ export class ChatSubmitAction extends SubmitAction {
 	static readonly ID = 'workbench.action.chat.submit';
 
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			ChatContextKeys.requestInProgress.negate(),
+			ChatContextKeys.location.notEqualsTo(ChatAgentLocation.EditingSession),
+		);
+
 		super({
 			id: ChatSubmitAction.ID,
 			title: localize2('interactive.submit.label', "Send and Dispatch"),
 			f1: false,
 			category: CHAT_CATEGORY,
 			icon: Codicon.send,
-			precondition: ContextKeyExpr.and(ChatContextKeys.inputHasText, ChatContextKeys.location.notEqualsTo(ChatAgentLocation.EditingSession)),
+			precondition,
 			keybinding: {
 				when: ChatContextKeys.inChatInput,
 				primary: KeyCode.Enter,
@@ -67,7 +78,10 @@ export class ChatSubmitAction extends SubmitAction {
 				{
 					id: MenuId.ChatExecute,
 					order: 4,
-					when: ContextKeyExpr.and(ChatContextKeys.requestInProgress.negate(), ChatContextKeys.location.notEqualsTo(ChatAgentLocation.EditingSession)),
+					when: ContextKeyExpr.and(
+						ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()),
+						ChatContextKeys.location.notEqualsTo(ChatAgentLocation.EditingSession),
+					),
 					group: 'navigation',
 				},
 			]
@@ -76,23 +90,26 @@ export class ChatSubmitAction extends SubmitAction {
 }
 
 export const ToggleAgentModeActionId = 'workbench.action.chat.toggleAgentMode';
+
+export interface IToggleAgentModeArgs {
+	agentMode: boolean;
+}
+
 export class ToggleAgentModeAction extends Action2 {
 	static readonly ID = ToggleAgentModeActionId;
 
 	constructor() {
 		super({
 			id: ToggleAgentModeAction.ID,
-			title: localize2('interactive.toggleAgent.label', "Toggle Agent Mode"),
+			title: localize2('interactive.toggleAgent.label', "Toggle Agent Mode (Experimental)"),
 			f1: true,
 			category: CHAT_CATEGORY,
 			precondition: ContextKeyExpr.and(
-				ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession),
-				ChatContextKeys.Editing.hasToolsAgent),
-			icon: Codicon.edit,
+				ChatContextKeys.Editing.hasToolsAgent,
+				ChatContextKeys.requestInProgress.negate()),
 			toggled: {
 				condition: ChatContextKeys.Editing.agentMode,
-				icon: Codicon.tools,
-				tooltip: localize('agentEnabled', "Agent Mode Enabled"),
+				tooltip: localize('agentEnabled', "Agent Mode Enabled (Experimental)"),
 			},
 			tooltip: localize('agentDisabled', "Agent Mode Disabled"),
 			keybinding: {
@@ -115,9 +132,81 @@ export class ToggleAgentModeAction extends Action2 {
 		});
 	}
 
-	override run(accessor: ServicesAccessor, ...args: any[]): void {
+	override async run(accessor: ServicesAccessor, ...args: any[]): Promise<void> {
 		const agentService = accessor.get(IChatAgentService);
-		agentService.toggleToolsAgentMode();
+		const chatEditingService = accessor.get(IChatEditingService);
+		const chatService = accessor.get(IChatService);
+		const commandService = accessor.get(ICommandService);
+		const dialogService = accessor.get(IDialogService);
+		const currentEditingSession = chatEditingService.currentEditingSession;
+		if (!currentEditingSession) {
+			return;
+		}
+
+		const entries = currentEditingSession.entries.get();
+		if (entries.length > 0 && entries.some(entry => entry.state.get() === WorkingSetEntryState.Modified)) {
+			if (!await discardAllEditsWithConfirmation(accessor)) {
+				// User cancelled
+				return;
+			}
+		} else {
+			const chatSession = chatService.getSession(currentEditingSession.chatSessionId);
+			if (chatSession?.getRequests().length) {
+				const confirmation = await dialogService.confirm({
+					title: localize('agent.newSession', "Start new session?"),
+					message: localize('agent.newSessionMessage', "Toggling agent mode will start a new session. Would you like to continue?"),
+					primaryButton: localize('agent.newSession.confirm', "Yes"),
+					type: 'info'
+				});
+				if (!confirmation.confirmed) {
+					return;
+				}
+			}
+		}
+
+		const arg = args[0] as IToggleAgentModeArgs | undefined;
+		agentService.toggleToolsAgentMode(typeof arg?.agentMode === 'boolean' ? arg.agentMode : undefined);
+
+		await commandService.executeCommand(ChatDoneActionId);
+	}
+}
+
+export const ToggleRequestPausedActionId = 'workbench.action.chat.toggleRequestPaused';
+export class ToggleRequestPausedAction extends Action2 {
+	static readonly ID = ToggleRequestPausedActionId;
+
+	constructor() {
+		super({
+			id: ToggleRequestPausedAction.ID,
+			title: localize2('interactive.toggleRequestPausd.label', "Toggle Request Paused"),
+			category: CHAT_CATEGORY,
+			icon: Codicon.debugPause,
+			toggled: {
+				condition: ChatContextKeys.isRequestPaused,
+				icon: Codicon.play,
+				tooltip: localize('requestIsPaused', "Resume Request"),
+			},
+			tooltip: localize('requestNotPaused', "Pause Request"),
+			menu: [
+				{
+					id: MenuId.ChatExecute,
+					order: 3.5,
+					when: ContextKeyExpr.and(
+						ChatContextKeys.canRequestBePaused,
+						ChatContextKeys.Editing.agentMode,
+						ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession),
+					),
+					group: 'navigation',
+				},
+			]
+		});
+	}
+
+	override run(accessor: ServicesAccessor, ...args: any[]): void {
+		const context: IChatExecuteActionContext | undefined = args[0];
+		const widgetService = accessor.get(IChatWidgetService);
+		const widget = context?.widget ?? widgetService.lastFocusedWidget;
+		widget?.togglePaused();
 	}
 }
 
@@ -125,13 +214,21 @@ export class ChatEditingSessionSubmitAction extends SubmitAction {
 	static readonly ID = 'workbench.action.edits.submit';
 
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			applyingChatEditsContextKey.toNegated(),
+			ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession),
+		);
+
 		super({
 			id: ChatEditingSessionSubmitAction.ID,
 			title: localize2('edits.submit.label', "Send"),
 			f1: false,
 			category: CHAT_CATEGORY,
 			icon: Codicon.send,
-			precondition: ContextKeyExpr.and(ChatContextKeys.inputHasText, ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey.toNegated()),
+			precondition,
 			keybinding: {
 				when: ChatContextKeys.inChatInput,
 				primary: KeyCode.Enter,
@@ -141,13 +238,13 @@ export class ChatEditingSessionSubmitAction extends SubmitAction {
 				{
 					id: MenuId.ChatExecuteSecondary,
 					group: 'group_1',
-					when: ContextKeyExpr.and(ChatContextKeys.requestInProgress.negate(), ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey.toNegated()),
+					when: ContextKeyExpr.and(ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()), ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey.toNegated()),
 					order: 1
 				},
 				{
 					id: MenuId.ChatExecute,
 					order: 4,
-					when: ContextKeyExpr.and(ChatContextKeys.requestInProgress.negate(), ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey.toNegated()),
+					when: ContextKeyExpr.and(ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()), ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey.toNegated()),
 					group: 'navigation',
 				},
 			]
@@ -159,18 +256,23 @@ class SubmitWithoutDispatchingAction extends Action2 {
 	static readonly ID = 'workbench.action.chat.submitWithoutDispatching';
 
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()),
+			ContextKeyExpr.and(ContextKeyExpr.or(
+				ChatContextKeys.location.isEqualTo(ChatAgentLocation.Panel),
+				ChatContextKeys.location.isEqualTo(ChatAgentLocation.Editor),
+			)),
+		);
+
 		super({
 			id: SubmitWithoutDispatchingAction.ID,
 			title: localize2('interactive.submitWithoutDispatch.label', "Send"),
 			f1: false,
 			category: CHAT_CATEGORY,
-			precondition: ContextKeyExpr.and(
-				ChatContextKeys.inputHasText,
-				ChatContextKeys.requestInProgress.negate(),
-				ContextKeyExpr.and(ContextKeyExpr.or(
-					ChatContextKeys.location.isEqualTo(ChatAgentLocation.Panel),
-					ChatContextKeys.location.isEqualTo(ChatAgentLocation.Editor),
-				))),
+			precondition,
 			keybinding: {
 				when: ChatContextKeys.inChatInput,
 				primary: KeyMod.Alt | KeyMod.Shift | KeyCode.Enter,
@@ -219,10 +321,18 @@ export class ChatSubmitSecondaryAgentAction extends Action2 {
 	static readonly ID = 'workbench.action.chat.submitSecondaryAgent';
 
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			ChatContextKeys.inputHasAgent.negate(),
+			ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()),
+		);
+
 		super({
 			id: ChatSubmitSecondaryAgentAction.ID,
 			title: localize2({ key: 'actions.chat.submitSecondaryAgent', comment: ['Send input from the chat input box to the secondary agent'] }, "Submit to Secondary Agent"),
-			precondition: ContextKeyExpr.and(ChatContextKeys.inputHasText, ChatContextKeys.inputHasAgent.negate(), ChatContextKeys.requestInProgress.negate()),
+			precondition,
 			menu: {
 				id: MenuId.ChatExecuteSecondary,
 				group: 'group_1',
@@ -262,10 +372,18 @@ export class ChatSubmitSecondaryAgentAction extends Action2 {
 
 class SendToChatEditingAction extends Action2 {
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			ChatContextKeys.inputHasAgent.negate(),
+			ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()),
+		);
+
 		super({
 			id: 'workbench.action.chat.sendToChatEditing',
 			title: localize2('chat.sendToChatEditing.label', "Send to Copilot Edits"),
-			precondition: ContextKeyExpr.and(ChatContextKeys.requestInProgress.negate(), ChatContextKeys.inputHasAgent.negate(), ChatContextKeys.inputHasText),
+			precondition,
 			category: CHAT_CATEGORY,
 			f1: false,
 			menu: {
@@ -347,10 +465,17 @@ class SendToChatEditingAction extends Action2 {
 
 class SendToNewChatAction extends Action2 {
 	constructor() {
+		const precondition = ContextKeyExpr.and(
+			// if the input has prompt instructions attached, allow submitting requests even
+			// without text present - having instructions is enough context for a request
+			ContextKeyExpr.or(ChatContextKeys.inputHasText, ChatContextKeys.instructionsAttached),
+			ContextKeyExpr.or(ChatContextKeys.isRequestPaused, ChatContextKeys.requestInProgress.negate()),
+		);
+
 		super({
 			id: 'workbench.action.chat.sendToNewChat',
 			title: localize2('chat.newChat.label', "Send to New Chat"),
-			precondition: ContextKeyExpr.and(ChatContextKeys.requestInProgress.negate(), ChatContextKeys.inputHasText),
+			precondition,
 			category: CHAT_CATEGORY,
 			f1: false,
 			menu: {
@@ -392,7 +517,10 @@ export class CancelAction extends Action2 {
 			icon: Codicon.stopCircle,
 			menu: {
 				id: MenuId.ChatExecute,
-				when: ContextKeyExpr.or(ChatContextKeys.requestInProgress, ContextKeyExpr.and(ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey)),
+				when: ContextKeyExpr.or(
+					ContextKeyExpr.and(ChatContextKeys.isRequestPaused.negate(), ChatContextKeys.requestInProgress),
+					ContextKeyExpr.and(ChatContextKeys.location.isEqualTo(ChatAgentLocation.EditingSession), applyingChatEditsContextKey)
+				),
 				order: 4,
 				group: 'navigation',
 			},
@@ -435,4 +563,5 @@ export function registerChatExecuteActions() {
 	registerAction2(ChatSubmitSecondaryAgentAction);
 	registerAction2(SendToChatEditingAction);
 	registerAction2(ToggleAgentModeAction);
+	registerAction2(ToggleRequestPausedAction);
 }
