@@ -52,7 +52,30 @@ export async function showEditsView(viewsService: IViewsService): Promise<IChatW
 	return (await viewsService.openView<ChatViewPane>(EditsViewId))?.widget;
 }
 
-export function ensureSideBarChatViewSize(width: number, viewDescriptorService: IViewDescriptorService, layoutService: IWorkbenchLayoutService): void {
+export function preferCopilotEditsView(viewsService: IViewsService): boolean {
+	if (viewsService.getFocusedView()?.id === ChatViewId || !!viewsService.getActiveViewWithId(ChatViewId)) {
+		return false;
+	}
+
+	return !!viewsService.getActiveViewWithId(EditsViewId);
+}
+
+export function showCopilotView(viewsService: IViewsService, layoutService: IWorkbenchLayoutService): Promise<IChatWidget | undefined> {
+
+	// Ensure main window is in front
+	if (layoutService.activeContainer !== layoutService.mainContainer) {
+		layoutService.mainContainer.focus();
+	}
+
+	// Bring up the correct view
+	if (preferCopilotEditsView(viewsService)) {
+		return showEditsView(viewsService);
+	} else {
+		return showChatView(viewsService);
+	}
+}
+
+export function ensureSideBarChatViewSize(viewDescriptorService: IViewDescriptorService, layoutService: IWorkbenchLayoutService): void {
 	const location = viewDescriptorService.getViewLocationById(ChatViewId);
 	if (location === ViewContainerLocation.Panel) {
 		return; // panel is typically very wide
@@ -60,8 +83,16 @@ export function ensureSideBarChatViewSize(width: number, viewDescriptorService:
 
 	const viewPart = location === ViewContainerLocation.Sidebar ? Parts.SIDEBAR_PART : Parts.AUXILIARYBAR_PART;
 	const partSize = layoutService.getSize(viewPart);
-	if (partSize.width < width) {
-		layoutService.setSize(viewPart, { width: width, height: partSize.height });
+
+	let adjustedChatWidth: number | undefined;
+	if (partSize.width < 400 && layoutService.mainContainerDimension.width > 1200) {
+		adjustedChatWidth = 400; // up to 400px if window bounds permit
+	} else if (partSize.width < 300) {
+		adjustedChatWidth = 300; // at minimum 300px
+	}
+
+	if (typeof adjustedChatWidth === 'number') {
+		layoutService.setSize(viewPart, { width: adjustedChatWidth, height: partSize.height });
 	}
 }
 
@@ -156,6 +187,7 @@ export interface IChatWidgetViewOptions {
 	defaultElementHeight?: number;
 	editorOverflowWidgetsDomNode?: HTMLElement;
 	enableImplicitContext?: boolean;
+	enableWorkingSet?: 'explicit' | 'implicit';
 }
 
 export interface IChatViewViewContext {
@@ -214,6 +246,7 @@ export interface IChatWidget {
 	getLastFocusedFileTreeForResponse(response: IChatResponseViewModel): IChatFileTreeInfo | undefined;
 	clear(): void;
 	getViewState(): IChatViewState;
+	togglePaused(): void;
 }
 
 
