@@ -6,12 +6,12 @@
 import './media/chatEditorController.css';
 import { addStandardDisposableListener, getTotalWidth } from '../../../../base/browser/dom.js';
 import { Disposable, DisposableStore, dispose, toDisposable } from '../../../../base/common/lifecycle.js';
-import { autorun, autorunWithStore, derived, IObservable, observableFromEvent, observableValue, transaction } from '../../../../base/common/observable.js';
+import { autorun, autorunWithStore, derived, IObservable, observableFromEvent, observableValue } from '../../../../base/common/observable.js';
 import { themeColorFromId } from '../../../../base/common/themables.js';
 import { ICodeEditor, IOverlayWidget, IOverlayWidgetPosition, IOverlayWidgetPositionCoordinates, IViewZone, MouseTargetType } from '../../../../editor/browser/editorBrowser.js';
 import { LineSource, renderLines, RenderOptions } from '../../../../editor/browser/widget/diffEditor/components/diffEditorViewZones/renderLines.js';
 import { diffAddDecoration, diffDeleteDecoration, diffWholeLineAddDecoration } from '../../../../editor/browser/widget/diffEditor/registrations.contribution.js';
-import { EditorOption, IEditorStickyScrollOptions } from '../../../../editor/common/config/editorOptions.js';
+import { EditorOption, IEditorOptions } from '../../../../editor/common/config/editorOptions.js';
 import { Range } from '../../../../editor/common/core/range.js';
 import { IDocumentDiff } from '../../../../editor/common/diff/documentDiffProvider.js';
 import { IEditorContribution, ScrollType } from '../../../../editor/common/editorCommon.js';
@@ -31,9 +31,17 @@ import { HiddenItemStrategy, MenuWorkbenchToolBar } from '../../../../platform/a
 import { observableCodeEditor } from '../../../../editor/browser/observableCodeEditor.js';
 import { minimapGutterAddedBackground, minimapGutterDeletedBackground, minimapGutterModifiedBackground, overviewRulerAddedForeground, overviewRulerDeletedForeground, overviewRulerModifiedForeground } from '../../scm/common/quickDiff.js';
 import { DetailedLineRangeMapping } from '../../../../editor/common/diff/rangeMapping.js';
+import { isDiffEditorForEntry } from './chatEditing/chatEditing.js';
+import { basename, isEqual } from '../../../../base/common/resources.js';
+import { ChatAgentLocation, IChatAgentService } from '../common/chatAgents.js';
+import { EditorsOrder, IEditorIdentifier, isDiffEditorInput } from '../../../common/editor.js';
+import { ChatEditorOverlayController } from './chatEditorOverlay.js';
+import { IChatService } from '../common/chatService.js';
 
+export const ctxIsGlobalEditingSession = new RawContextKey<boolean>('chat.isGlobalEditingSession', undefined, localize('chat.ctxEditSessionIsGlobal', "The current editor is part of the global edit session"));
 export const ctxHasEditorModification = new RawContextKey<boolean>('chat.hasEditorModifications', undefined, localize('chat.hasEditorModifications', "The current editor contains chat modifications"));
 export const ctxHasRequestInProgress = new RawContextKey<boolean>('chat.ctxHasRequestInProgress', false, localize('chat.ctxHasRequestInProgress', "The current editor shows a file from an edit session which is still in progress"));
+export const ctxReviewModeEnabled = new RawContextKey<boolean>('chat.ctxReviewModeEnabled', true, localize('chat.ctxReviewModeEnabled', "Review mode for chat changes is enabled"));
 
 export class ChatEditorController extends Disposable implements IEditorContribution {
 
@@ -47,8 +55,13 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 	private readonly _diffHunkWidgets: DiffHunkWidget[] = [];
 
 	private _viewZones: string[] = [];
+
+	private readonly _overlayCtrl: ChatEditorOverlayController;
+
+	private readonly _ctxIsGlobalEditsSession: IContextKey<boolean>;
 	private readonly _ctxHasEditorModification: IContextKey<boolean>;
 	private readonly _ctxRequestInProgress: IContextKey<boolean>;
+	private readonly _ctxReviewModelEnabled: IContextKey<boolean>;
 
 	static get(editor: ICodeEditor): ChatEditorController | null {
 		const controller = editor.getContribution<ChatEditorController>(ChatEditorController.ID);
@@ -65,38 +78,54 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 
 	constructor(
 		private readonly _editor: ICodeEditor,
-		@IInstantiationService private readonly _instantiationService: IInstantiationService,
 		@IChatEditingService private readonly _chatEditingService: IChatEditingService,
+		@IInstantiationService private readonly _instantiationService: IInstantiationService,
+		@IChatAgentService private readonly _chatAgentService: IChatAgentService,
 		@IEditorService private readonly _editorService: IEditorService,
 		@IContextKeyService contextKeyService: IContextKeyService,
+		@IChatService chatService: IChatService,
 	) {
 		super();
 
+		this._overlayCtrl = ChatEditorOverlayController.get(_editor)!;
+		this._ctxIsGlobalEditsSession = ctxIsGlobalEditingSession.bindTo(contextKeyService);
 		this._ctxHasEditorModification = ctxHasEditorModification.bindTo(contextKeyService);
 		this._ctxRequestInProgress = ctxHasRequestInProgress.bindTo(contextKeyService);
+		this._ctxReviewModelEnabled = ctxReviewModeEnabled.bindTo(contextKeyService);
 
-		const fontInfoObs = observableCodeEditor(this._editor).getOption(EditorOption.fontInfo);
-		const lineHeightObs = observableCodeEditor(this._editor).getOption(EditorOption.lineHeight);
-		const modelObs = observableCodeEditor(this._editor).model;
-
+		const editorObs = observableCodeEditor(this._editor);
+		const fontInfoObs = editorObs.getOption(EditorOption.fontInfo);
+		const lineHeightObs = editorObs.getOption(EditorOption.lineHeight);
+		const modelObs = editorObs.model;
 
 		this._store.add(autorun(r => {
-			const session = this._chatEditingService.currentEditingSessionObs.read(r);
-			this._ctxRequestInProgress.set(session?.state.read(r) === ChatEditingSessionState.StreamingEdits);
+			let isStreamingEdits = false;
+			for (const session of _chatEditingService.editingSessionsObs.read(r)) {
+				isStreamingEdits ||= session.state.read(r) === ChatEditingSessionState.StreamingEdits;
+			}
+			this._ctxRequestInProgress.set(isStreamingEdits);
 		}));
 
-
 		const entryForEditor = derived(r => {
 			const model = modelObs.read(r);
-			const session = this._chatEditingService.currentEditingSessionObs.read(r);
-			if (!session) {
-				return undefined;
+			if (!model) {
+				return;
 			}
-			const entry = model?.uri ? session.readEntry(model.uri, r) : undefined;
-			if (!entry || entry.state.read(r) !== WorkingSetEntryState.Modified) {
-				return undefined;
+
+			for (const session of _chatEditingService.editingSessionsObs.read(r)) {
+				const entries = session.entries.read(r);
+				const idx = model?.uri
+					? entries.findIndex(e => isEqual(e.modifiedURI, model.uri))
+					: -1;
+
+				const chatModel = chatService.getSession(session.chatSessionId);
+
+				if (idx >= 0 && chatModel) {
+					return { session, chatModel, entry: entries[idx], entries, idx };
+				}
 			}
-			return { session, entry };
+
+			return undefined;
 		});
 
 
@@ -104,24 +133,49 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 
 		this._register(autorunWithStore((r, store) => {
 
-			if (this._editor.getOption(EditorOption.inDiffEditor)) {
-				this._clearRendering();
-				return;
-			}
-
 			const currentEditorEntry = entryForEditor.read(r);
 
 			if (!currentEditorEntry) {
-				this._clearRendering();
+				this._ctxIsGlobalEditsSession.reset();
+				this._clear();
 				didReval = false;
 				return;
 			}
 
-			const { session, entry } = currentEditorEntry;
+			if (this._editor.getOption(EditorOption.inDiffEditor) && !_instantiationService.invokeFunction(isDiffEditorForEntry, currentEditorEntry.entry, this._editor)) {
+				this._clear();
+				return;
+			}
 
-			const entryIndex = session.entries.read(r).indexOf(entry);
-			this._currentEntryIndex.set(entryIndex, undefined);
+			const { session, chatModel, entries, idx, entry } = currentEditorEntry;
 
+			store.add(chatModel.onDidChange(e => {
+				if (e.kind === 'addRequest') {
+					didReval = false;
+				}
+			}));
+
+			this._ctxIsGlobalEditsSession.set(session.isGlobalEditingSession);
+			this._ctxReviewModelEnabled.set(entry.reviewMode.read(r));
+
+			// context
+			this._currentEntryIndex.set(idx, undefined);
+
+			// overlay widget
+			if (entry.state.read(r) !== WorkingSetEntryState.Modified) {
+				this._overlayCtrl.hide();
+			} else {
+				this._overlayCtrl.showEntry(
+					session,
+					entry, entries[(idx + 1) % entries.length],
+					{
+						entryIndex: this._currentEntryIndex,
+						changeIndex: this._currentChangeIndex
+					}
+				);
+			}
+
+			// scrolling logic
 			if (entry.isCurrentlyBeingModified.read(r)) {
 				// while modified: scroll along unless locked
 				if (!this._scrollLock) {
@@ -140,11 +194,22 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 				lineHeightObs.read(r);
 
 				const diff = entry?.diffInfo.read(r);
-				this._updateWithDiff(entry, diff);
+
+				// Add line decorations (just markers, no UI) for diff navigation
+				this._updateDiffLineDecorations(diff);
+
+				const reviewMode = entry.reviewMode.read(r);
+
+				// Add diff decoration to the UI (unless in diff editor)
+				if (!this._editor.getOption(EditorOption.inDiffEditor)) {
+					this._updateDiffRendering(entry, diff, reviewMode);
+				} else {
+					this._clearDiffRendering();
+				}
 
 				if (!didReval && !diff.identical) {
 					didReval = true;
-					this.revealNext();
+					this._reveal(true, false, ScrollType.Immediate);
 				}
 			}
 		}));
@@ -152,26 +217,31 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		// ---- readonly while streaming
 
 		const shouldBeReadOnly = derived(this, r => {
-			const value = this._chatEditingService.currentEditingSessionObs.read(r);
-			if (!value || value.state.read(r) !== ChatEditingSessionState.StreamingEdits) {
-				return false;
-			}
 
 			const model = modelObs.read(r);
-			return model ? value.readEntry(model.uri, r) : undefined;
+			if (!model) {
+				return undefined;
+			}
+			for (const session of _chatEditingService.editingSessionsObs.read(r)) {
+				if (session.readEntry(model.uri, r) && session.state.read(r) === ChatEditingSessionState.StreamingEdits) {
+					return true;
+				}
+			}
+			return false;
 		});
 
 
-		let actualReadonly: boolean | undefined;
-		let actualDeco: 'off' | 'editable' | 'on' | undefined;
-		let actualStickyScroll: IEditorStickyScrollOptions | undefined;
+		let actualOptions: IEditorOptions | undefined;
 
 		this._register(autorun(r => {
 			const value = shouldBeReadOnly.read(r);
 			if (value) {
-				actualReadonly ??= this._editor.getOption(EditorOption.readOnly);
-				actualDeco ??= this._editor.getOption(EditorOption.renderValidationDecorations);
-				actualStickyScroll ??= this._editor.getOption(EditorOption.stickyScroll);
+
+				actualOptions ??= {
+					readOnly: this._editor.getOption(EditorOption.readOnly),
+					renderValidationDecorations: this._editor.getOption(EditorOption.renderValidationDecorations),
+					stickyScroll: this._editor.getOption(EditorOption.stickyScroll)
+				};
 
 				this._editor.updateOptions({
 					readOnly: true,
@@ -179,26 +249,30 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 					stickyScroll: { enabled: false }
 				});
 			} else {
-				if (actualReadonly !== undefined && actualDeco !== undefined && actualStickyScroll !== undefined) {
-					this._editor.updateOptions({
-						readOnly: actualReadonly,
-						renderValidationDecorations: actualDeco,
-						stickyScroll: actualStickyScroll
-					});
-					actualReadonly = undefined;
-					actualDeco = undefined;
-					actualStickyScroll = undefined;
+				if (actualOptions !== undefined) {
+					this._editor.updateOptions(actualOptions);
+					actualOptions = undefined;
 				}
 			}
 		}));
 	}
 
 	override dispose(): void {
-		this._clearRendering();
+		this._clear();
 		super.dispose();
 	}
 
-	private _clearRendering() {
+	private _clear() {
+		this._clearDiffRendering();
+		this._overlayCtrl.hide();
+		this._diffLineDecorations.clear();
+		this._currentChangeIndex.set(undefined, undefined);
+		this._currentEntryIndex.set(undefined, undefined);
+		this._ctxHasEditorModification.reset();
+		this._ctxReviewModelEnabled.reset();
+	}
+
+	private _clearDiffRendering() {
 		this._editor.changeViewZones((viewZoneChangeAccessor) => {
 			for (const id of this._viewZones) {
 				viewZoneChangeAccessor.removeZone(id);
@@ -207,18 +281,11 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		this._viewZones = [];
 		this._diffHunksRenderStore.clear();
 		this._diffVisualDecorations.clear();
-		this._diffLineDecorations.clear();
-		this._ctxHasEditorModification.reset();
-		transaction(tx => {
-			this._currentEntryIndex.set(undefined, tx);
-			this._currentChangeIndex.set(undefined, tx);
-		});
 		this._scrollLock = false;
 	}
 
-	private _updateWithDiff(entry: IModifiedFileEntry, diff: IDocumentDiff): void {
+	private _updateDiffRendering(entry: IModifiedFileEntry, diff: IDocumentDiff, reviewMode: boolean): void {
 
-		this._ctxHasEditorModification.set(!diff.identical);
 		const originalModel = entry.originalModel;
 
 		const chatDiffAddDecoration = ModelDecorationOptions.createDynamic({
@@ -250,7 +317,6 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 			}
 			this._viewZones = [];
 			const modifiedVisualDecorations: IModelDeltaDecoration[] = [];
-			const modifiedLineDecorations: IModelDeltaDecoration[] = [];
 			const mightContainNonBasicASCII = originalModel.mightContainNonBasicASCII();
 			const mightContainRTL = originalModel.mightContainRTL();
 			const renderOptions = RenderOptions.fromEditor(this._editor);
@@ -267,18 +333,21 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 					mightContainRTL,
 				);
 				const decorations: InlineDecoration[] = [];
-				for (const i of diffEntry.innerChanges || []) {
-					decorations.push(new InlineDecoration(
-						i.originalRange.delta(-(diffEntry.original.startLineNumber - 1)),
-						diffDeleteDecoration.className!,
-						InlineDecorationType.Regular
-					));
 
-					// If the original range is empty, the start line number is 1 and the new range spans the entire file, don't draw an Added decoration
-					if (!(i.originalRange.isEmpty() && i.originalRange.startLineNumber === 1 && i.modifiedRange.endLineNumber === editorLineCount) && !i.modifiedRange.isEmpty()) {
-						modifiedVisualDecorations.push({
-							range: i.modifiedRange, options: chatDiffAddDecoration
-						});
+				if (reviewMode) {
+					for (const i of diffEntry.innerChanges || []) {
+						decorations.push(new InlineDecoration(
+							i.originalRange.delta(-(diffEntry.original.startLineNumber - 1)),
+							diffDeleteDecoration.className!,
+							InlineDecorationType.Regular
+						));
+
+						// If the original range is empty, the start line number is 1 and the new range spans the entire file, don't draw an Added decoration
+						if (!(i.originalRange.isEmpty() && i.originalRange.startLineNumber === 1 && i.modifiedRange.endLineNumber === editorLineCount) && !i.modifiedRange.isEmpty()) {
+							modifiedVisualDecorations.push({
+								range: i.modifiedRange, options: chatDiffAddDecoration
+							});
+						}
 					}
 				}
 
@@ -312,43 +381,41 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 						options: modifiedDecoration
 					});
 				}
-				const domNode = document.createElement('div');
-				domNode.className = 'chat-editing-original-zone view-lines line-delete monaco-mouse-cursor-text';
-				const result = renderLines(source, renderOptions, decorations, domNode);
 
-				if (!isCreatedContent) {
-					const viewZoneData: IViewZone = {
-						afterLineNumber: diffEntry.modified.startLineNumber - 1,
-						heightInLines: result.heightInLines,
-						domNode,
-						ordinal: 50000 + 2 // more than https://github.com/microsoft/vscode/blob/bf52a5cfb2c75a7327c9adeaefbddc06d529dcad/src/vs/workbench/contrib/inlineChat/browser/inlineChatZoneWidget.ts#L42
-					};
+				if (reviewMode) {
+					const domNode = document.createElement('div');
+					domNode.className = 'chat-editing-original-zone view-lines line-delete monaco-mouse-cursor-text';
+					const result = renderLines(source, renderOptions, decorations, domNode);
 
-					this._viewZones.push(viewZoneChangeAccessor.addZone(viewZoneData));
-				}
+					if (!isCreatedContent) {
 
-				// Add content widget for each diff change
-				const widget = this._instantiationService.createInstance(DiffHunkWidget, entry, diffEntry, this._editor.getModel()!.getVersionId(), this._editor, isCreatedContent ? 0 : result.heightInLines);
-				widget.layout(diffEntry.modified.startLineNumber);
+						const viewZoneData: IViewZone = {
+							afterLineNumber: diffEntry.modified.startLineNumber - 1,
+							heightInLines: result.heightInLines,
+							domNode,
+							ordinal: 50000 + 2 // more than https://github.com/microsoft/vscode/blob/bf52a5cfb2c75a7327c9adeaefbddc06d529dcad/src/vs/workbench/contrib/inlineChat/browser/inlineChatZoneWidget.ts#L42
+						};
 
-				this._diffHunkWidgets.push(widget);
-				diffHunkDecorations.push({
-					range: diffEntry.modified.toInclusiveRange() ?? new Range(diffEntry.modified.startLineNumber, 1, diffEntry.modified.startLineNumber, Number.MAX_SAFE_INTEGER),
-					options: {
-						description: 'diff-hunk-widget',
-						stickiness: TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges
+						this._viewZones.push(viewZoneChangeAccessor.addZone(viewZoneData));
 					}
-				});
 
-				// Add line decorations for diff navigation
-				modifiedLineDecorations.push({
-					range: diffEntry.modified.toInclusiveRange() ?? new Range(diffEntry.modified.startLineNumber, 1, diffEntry.modified.startLineNumber, Number.MAX_SAFE_INTEGER),
-					options: ChatEditorController._diffLineDecorationData
-				});
+
+					// Add content widget for each diff change
+					const widget = this._instantiationService.createInstance(DiffHunkWidget, entry, diffEntry, this._editor.getModel()!.getVersionId(), this._editor, isCreatedContent ? 0 : result.heightInLines);
+					widget.layout(diffEntry.modified.startLineNumber);
+
+					this._diffHunkWidgets.push(widget);
+					diffHunkDecorations.push({
+						range: diffEntry.modified.toInclusiveRange() ?? new Range(diffEntry.modified.startLineNumber, 1, diffEntry.modified.startLineNumber, Number.MAX_SAFE_INTEGER),
+						options: {
+							description: 'diff-hunk-widget',
+							stickiness: TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges
+						}
+					});
+				}
 			}
 
 			this._diffVisualDecorations.set(modifiedVisualDecorations);
-			this._diffLineDecorations.set(modifiedLineDecorations);
 		});
 
 		const diffHunkDecoCollection = this._editor.createDecorationsCollection(diffHunkDecorations);
@@ -423,6 +490,20 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		}));
 	}
 
+	private _updateDiffLineDecorations(diff: IDocumentDiff): void {
+		this._ctxHasEditorModification.set(!diff.identical);
+
+		const modifiedLineDecorations: IModelDeltaDecoration[] = [];
+
+		for (const diffEntry of diff.changes) {
+			modifiedLineDecorations.push({
+				range: diffEntry.modified.toInclusiveRange() ?? new Range(diffEntry.modified.startLineNumber, 1, diffEntry.modified.startLineNumber, Number.MAX_SAFE_INTEGER),
+				options: ChatEditorController._diffLineDecorationData
+			});
+		}
+		this._diffLineDecorations.set(modifiedLineDecorations);
+	}
+
 	unlockScroll(): void {
 		this._scrollLock = false;
 	}
@@ -441,7 +522,7 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		return this._reveal(false, strict);
 	}
 
-	private _reveal(next: boolean, strict: boolean): boolean {
+	private _reveal(next: boolean, strict: boolean, scrollType = ScrollType.Smooth): boolean {
 		const position = this._editor.getPosition();
 		if (!position) {
 			this._currentChangeIndex.set(undefined, undefined);
@@ -480,7 +561,7 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 
 		const targetPosition = next ? decorations[target].getStartPosition() : decorations[target].getEndPosition();
 		this._editor.setPosition(targetPosition);
-		this._editor.revealPositionInCenter(targetPosition, ScrollType.Smooth);
+		this._editor.revealPositionInCenter(targetPosition, scrollType);
 		this._editor.focus();
 
 		return true;
@@ -508,7 +589,7 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		return closestWidget;
 	}
 
-	undoNearestChange(closestWidget: DiffHunkWidget | undefined): void {
+	rejectNearestChange(closestWidget: DiffHunkWidget | undefined): void {
 		closestWidget = closestWidget ?? this._findClosestWidget();
 		if (closestWidget instanceof DiffHunkWidget) {
 			closestWidget.reject();
@@ -524,10 +605,23 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 		}
 	}
 
-	async openDiff(widget: DiffHunkWidget | undefined): Promise<void> {
+	async toggleDiff(widget: DiffHunkWidget | undefined): Promise<void> {
 		if (!this._editor.hasModel()) {
 			return;
 		}
+
+		let entry: IModifiedFileEntry | undefined;
+		for (const session of this._chatEditingService.editingSessionsObs.get()) {
+			entry = session.getEntry(this._editor.getModel().uri);
+			if (entry) {
+				break;
+			}
+		}
+
+		if (!entry) {
+			return;
+		}
+
 		const lineRelativeTop = this._editor.getTopForLineNumber(this._editor.getPosition().lineNumber) - this._editor.getScrollTop();
 		let closestDistance = Number.MAX_VALUE;
 
@@ -544,22 +638,57 @@ export class ChatEditorController extends Disposable implements IEditorContribut
 			}
 		}
 
+		let selection = this._editor.getSelection();
 		if (widget instanceof DiffHunkWidget) {
-
 			const lineNumber = widget.getStartLineNumber();
 			const position = lineNumber ? new Position(lineNumber, 1) : undefined;
-			let selection = this._editor.getSelection();
 			if (position && !selection.containsPosition(position)) {
 				selection = Selection.fromPositions(position);
 			}
+		}
 
+		const isDiffEditor = this._editor.getOption(EditorOption.inDiffEditor);
+
+		if (isDiffEditor) {
+			// normal EDITOR
+			await this._editorService.openEditor({ resource: entry.modifiedURI });
+
+		} else {
+			// DIFF editor
+			const defaultAgentName = this._chatAgentService.getDefaultAgent(ChatAgentLocation.EditingSession)?.fullName;
 			const diffEditor = await this._editorService.openEditor({
-				original: { resource: widget.entry.originalURI, options: { selection: undefined } },
-				modified: { resource: widget.entry.modifiedURI, options: { selection } },
+				original: { resource: entry.originalURI, options: { selection: undefined } },
+				modified: { resource: entry.modifiedURI, options: { selection } },
+				label: defaultAgentName
+					? localize('diff.agent', '{0} (changes from {1})', basename(entry.modifiedURI), defaultAgentName)
+					: localize('diff.generic', '{0} (changes from chat)', basename(entry.modifiedURI))
 			});
 
-			// this is needed, passing the selection doesn't seem to work
-			diffEditor?.getControl()?.setSelection(selection);
+			if (diffEditor && diffEditor.input) {
+
+				// this is needed, passing the selection doesn't seem to work
+				diffEditor.getControl()?.setSelection(selection);
+
+				// close diff editor when entry is decided
+				const d = autorun(r => {
+					const state = entry.state.read(r);
+					if (state === WorkingSetEntryState.Accepted || state === WorkingSetEntryState.Rejected) {
+						d.dispose();
+
+						const editorIdents: IEditorIdentifier[] = [];
+						for (const candidate of this._editorService.getEditors(EditorsOrder.MOST_RECENTLY_ACTIVE)) {
+							if (isDiffEditorInput(candidate.editor)
+								&& isEqual(candidate.editor.original.resource, entry.originalURI)
+								&& isEqual(candidate.editor.modified.resource, entry.modifiedURI)
+							) {
+								editorIdents.push(candidate);
+							}
+						}
+
+						this._editorService.closeEditors(editorIdents);
+					}
+				});
+			}
 		}
 	}
 }
@@ -597,6 +726,7 @@ class DiffHunkWidget implements IOverlayWidget {
 		});
 
 		this._store.add(toolbar);
+		this._store.add(toolbar.actionRunner.onWillRun(_ => _editor.focus()));
 		this._editor.addOverlayWidget(this);
 	}
 
