@@ -15,7 +15,7 @@ import { MarkdownString } from '../../../../base/common/htmlContent.js';
 import { Disposable, DisposableStore, IDisposable, MutableDisposable, combinedDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
 import { ResourceSet } from '../../../../base/common/map.js';
 import { Schemas } from '../../../../base/common/network.js';
-import { autorun } from '../../../../base/common/observable.js';
+import { autorunWithStore, observableFromEvent } from '../../../../base/common/observable.js';
 import { extUri, isEqual } from '../../../../base/common/resources.js';
 import { isDefined } from '../../../../base/common/types.js';
 import { URI } from '../../../../base/common/uri.js';
@@ -38,7 +38,7 @@ import { IThemeService } from '../../../../platform/theme/common/themeService.js
 import { ChatAgentLocation, IChatAgentCommand, IChatAgentData, IChatAgentService, IChatWelcomeMessageContent, isChatWelcomeMessageContent } from '../common/chatAgents.js';
 import { ChatContextKeys } from '../common/chatContextKeys.js';
 import { IChatEditingService, IChatEditingSession, WorkingSetEntryRemovalReason, WorkingSetEntryState } from '../common/chatEditingService.js';
-import { IChatModel, IChatRequestVariableEntry, IChatResponseModel } from '../common/chatModel.js';
+import { ChatPauseState, IChatModel, IChatRequestVariableEntry, IChatResponseModel } from '../common/chatModel.js';
 import { ChatRequestAgentPart, IParsedChatRequest, chatAgentLeader, chatSubcommandLeader, formatChatQuestion } from '../common/chatParserTypes.js';
 import { ChatRequestParser } from '../common/chatRequestParser.js';
 import { IChatFollowup, IChatLocationData, IChatSendRequestOptions, IChatService } from '../common/chatService.js';
@@ -151,6 +151,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	private bodyDimension: dom.Dimension | undefined;
 	private visibleChangeCount = 0;
 	private requestInProgress: IContextKey<boolean>;
+	private isRequestPaused: IContextKey<boolean>;
+	private canRequestBePaused: IContextKey<boolean>;
 	private agentInInput: IContextKey<boolean>;
 
 	private _visible = false;
@@ -187,6 +189,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		return this._viewModel;
 	}
 
+	private _editingSession: IChatEditingSession | undefined;
+
 	private parsedChatRequest: IParsedChatRequest | undefined;
 	get parsedInput() {
 		if (this.parsedChatRequest === undefined) {
@@ -246,41 +250,47 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		ChatContextKeys.inQuickChat.bindTo(contextKeyService).set(isQuickChat(this));
 		this.agentInInput = ChatContextKeys.inputHasAgent.bindTo(contextKeyService);
 		this.requestInProgress = ChatContextKeys.requestInProgress.bindTo(contextKeyService);
+		this.isRequestPaused = ChatContextKeys.isRequestPaused.bindTo(contextKeyService);
+		this.canRequestBePaused = ChatContextKeys.canRequestBePaused.bindTo(contextKeyService);
 
 		this._codeBlockModelCollection = this._register(instantiationService.createInstance(CodeBlockModelCollection));
 
-		const chatEditingSessionDisposables = this._register(new DisposableStore());
 
-		this._register(autorun(r => {
-			const session = this.chatEditingService.currentEditingSessionObs.read(r);
+		const viewModelObs = observableFromEvent(this, this.onDidChangeViewModel, () => this.viewModel);
+
+		this._register(autorunWithStore((r, store) => {
+
+			const viewModel = viewModelObs.read(r);
+			const sessions = chatEditingService.editingSessionsObs.read(r);
+
+			const session = sessions.find(candidate => candidate.chatSessionId === viewModel?.sessionId);
+			this._editingSession = undefined;
+			this.renderChatEditingSessionState(); // this is necessary to make sure we dispose previous buttons, etc.
+
 			if (!session) {
+				// none or for a different chat widget
 				return;
 			}
-			if (session.chatSessionId !== this.viewModel?.sessionId) {
-				// this chat editing session is for a different chat widget
-				return;
-			}
-			// make sure to clean up anything related to the prev session (if any)
-			chatEditingSessionDisposables.clear();
-			this.renderChatEditingSessionState(null); // this is necessary to make sure we dispose previous buttons, etc.
 
-			chatEditingSessionDisposables.add(session.onDidChange(() => {
-				this.renderChatEditingSessionState(session);
+			this._editingSession = session;
+
+			store.add(session.onDidChange(() => {
+				this.renderChatEditingSessionState();
 			}));
-			chatEditingSessionDisposables.add(session.onDidDispose(() => {
-				chatEditingSessionDisposables.clear();
-				this.renderChatEditingSessionState(null);
+			store.add(session.onDidDispose(() => {
+				this._editingSession = undefined;
+				this.renderChatEditingSessionState();
 			}));
-			chatEditingSessionDisposables.add(this.onDidChangeParsedInput(() => {
-				this.renderChatEditingSessionState(session);
+			store.add(this.onDidChangeParsedInput(() => {
+				this.renderChatEditingSessionState();
 			}));
-			chatEditingSessionDisposables.add(this.inputEditor.onDidChangeModelContent(() => {
+			store.add(this.inputEditor.onDidChangeModelContent(() => {
 				if (this.getInput() === '') {
 					this.refreshParsedInput();
-					this.renderChatEditingSessionState(session);
+					this.renderChatEditingSessionState();
 				}
 			}));
-			this.renderChatEditingSessionState(session);
+			this.renderChatEditingSessionState();
 		}));
 
 		if (this._location.location === ChatAgentLocation.EditingSession) {
@@ -289,7 +299,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				const sessionId = this._viewModel?.sessionId;
 				if (sessionId) {
 					if (sessionId !== currentEditSession?.chatSessionId) {
-						currentEditSession = await this.chatEditingService.startOrContinueEditingSession(sessionId);
+						currentEditSession = await chatEditingService.startOrContinueEditingSession(sessionId);
 					}
 				} else {
 					if (currentEditSession) {
@@ -561,8 +571,14 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	}
 
 	private renderWelcomeViewContentIfNeeded() {
+		if (this.viewOptions.renderStyle === 'compact' || this.viewOptions.renderStyle === 'minimal') {
+			return;
+		}
+
+		const numItems = this.viewModel?.getItems().length ?? 0;
 		const welcomeContent = this.viewModel?.model.welcomeMessage ?? this.persistedWelcomeMessage;
-		if (welcomeContent && this.welcomeMessageContainer.children.length === 0 && !this.viewOptions.renderStyle) {
+		if (welcomeContent && !numItems && (this.welcomeMessageContainer.children.length === 0 || this.location === ChatAgentLocation.EditingSession)) {
+			dom.clearNode(this.welcomeMessageContainer);
 			const tips = this.viewOptions.supportsAdditionalParticipants
 				? new MarkdownString(localize('chatWidget.tips', "{0} or type {1} to attach context\n\n{2} to chat with extensions\n\nType {3} to use commands", '$(attach)', '#', '$(mention)', '/'), { supportThemeIcons: true })
 				: new MarkdownString(localize('chatWidget.tips.withoutParticipants', "{0} or type {1} to attach context", '$(attach)', '#'), { supportThemeIcons: true });
@@ -574,15 +590,17 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			dom.append(this.welcomeMessageContainer, welcomePart.element);
 		}
 
-		if (!this.viewOptions.renderStyle && this.viewModel) {
-			const treeItems = this.viewModel.getItems();
-			dom.setVisibility(treeItems.length === 0, this.welcomeMessageContainer);
-			dom.setVisibility(treeItems.length !== 0, this.listContainer);
+		if (this.viewModel) {
+			dom.setVisibility(numItems === 0, this.welcomeMessageContainer);
+			dom.setVisibility(numItems !== 0, this.listContainer);
 		}
 	}
 
-	private async renderChatEditingSessionState(session: IChatEditingSession | null) {
-		this.inputPart.renderChatEditingSessionState(session, this);
+	private async renderChatEditingSessionState() {
+		if (!this.inputPart) {
+			return;
+		}
+		this.inputPart.renderChatEditingSessionState(this._editingSession ?? null, this);
 
 		if (this.bodyDimension) {
 			this.layout(this.bodyDimension.height, this.bodyDimension.width);
@@ -650,7 +668,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					noCommandDetection: true,
 					attempt: request.attempt + 1,
 					location: this.location,
-					userSelectedModelId: this.input.currentLanguageModel
+					userSelectedModelId: this.input.currentLanguageModel,
+					hasInstructionAttachments: this.input.hasInstructionAttachments,
 				};
 				this.chatService.resendRequest(request, options).catch(e => this.logService.error('FAILED to rerun request', e));
 			}
@@ -672,6 +691,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				keyboardNavigationLabelProvider: { getKeyboardNavigationLabel: (e: ChatTreeItem) => isRequestVM(e) ? e.message : isResponseVM(e) ? e.response.value : '' }, // TODO
 				setRowLineHeight: false,
 				filter: this.viewOptions.filter ? { filter: this.viewOptions.filter.bind(this.viewOptions), } : undefined,
+				scrollToActiveElement: true,
 				overrideStyles: {
 					listFocusBackground: this.styles.listBackground,
 					listInactiveFocusBackground: this.styles.listBackground,
@@ -761,7 +781,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				renderStyle: options?.renderStyle === 'minimal' ? 'compact' : options?.renderStyle,
 				menus: { executeToolbar: MenuId.ChatExecute, ...this.viewOptions.menus },
 				editorOverflowWidgetsDomNode: this.viewOptions.editorOverflowWidgetsDomNode,
-				enableImplicitContext: this.viewOptions.enableImplicitContext
+				enableImplicitContext: this.viewOptions.enableImplicitContext,
+				renderWorkingSet: this.viewOptions.enableWorkingSet === 'explicit'
 			},
 			this.styles,
 			() => this.collectInputState()
@@ -827,16 +848,21 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			this._onDidChangeContentHeight.fire();
 		}));
 		this._register(this.inputPart.attachmentModel.onDidChangeContext(() => {
-			if (this.chatEditingService.currentEditingSession && this.chatEditingService.currentEditingSession?.chatSessionId === this.viewModel?.sessionId) {
+			if (this._editingSession) {
 				// TODO still needed? Do this inside input part and fire onDidChangeHeight?
-				this.renderChatEditingSessionState(this.chatEditingService.currentEditingSession);
+				this.renderChatEditingSessionState();
 			}
 		}));
 		this._register(this.inputEditor.onDidChangeModelContent(() => {
 			this.parsedChatRequest = undefined;
 			this.updateChatInputContext();
 		}));
-		this._register(this.chatAgentService.onDidChangeAgents(() => this.parsedChatRequest = undefined));
+		this._register(this.chatAgentService.onDidChangeAgents(() => {
+			this.parsedChatRequest = undefined;
+
+			// Tools agent loads -> welcome content changes
+			this.renderWelcomeViewContentIfNeeded();
+		}));
 	}
 
 	private onDidStyleChange(): void {
@@ -845,6 +871,10 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this.container.style.setProperty('--vscode-chat-list-background', this.themeService.getColorTheme().getColor(this.styles.listBackground)?.toString() ?? '');
 	}
 
+	togglePaused() {
+		this.viewModel?.model.toggleLastRequestPaused();
+	}
+
 	setModel(model: IChatModel, viewState: IChatViewState): void {
 		if (!this.container) {
 			throw new Error('Call render() before setModel()');
@@ -864,14 +894,16 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			}
 
 			this.requestInProgress.set(this.viewModel.requestInProgress);
+			this.isRequestPaused.set(this.viewModel.requestPausibility === ChatPauseState.Paused);
+			this.canRequestBePaused.set(this.viewModel.requestPausibility !== ChatPauseState.NotPausable);
 
 			this.onDidChangeItems();
 			if (events.some(e => e?.kind === 'addRequest') && this.visible) {
 				this.scrollToEnd();
 			}
 
-			if (this.chatEditingService.currentEditingSession && this.chatEditingService.currentEditingSession?.chatSessionId === this.viewModel?.sessionId) {
-				this.renderChatEditingSessionState(this.chatEditingService.currentEditingSession);
+			if (this._editingSession) {
+				this.renderChatEditingSessionState();
 			}
 		}));
 		this.viewModelDisposables.add(this.viewModel.onDidDisposeModel(() => {
@@ -985,7 +1017,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	}
 
 	private async _acceptInput(query: { query: string } | { prefix: string } | undefined, options?: IChatAcceptInputOptions): Promise<IChatResponseModel | undefined> {
-		if (this.viewModel?.requestInProgress) {
+		if (this.viewModel?.requestInProgress && this.viewModel.requestPausibility !== ChatPauseState.Paused) {
 			return;
 		}
 
@@ -1002,25 +1034,35 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					`${query.prefix} ${editorValue}`;
 			const isUserQuery = !query || 'prefix' in query;
 
-
+			const { promptInstructions } = this.inputPart.attachmentModel;
+			const instructionsEnabled = promptInstructions.featureEnabled;
+			if (instructionsEnabled) {
+				// instruction files may have nested child references to other prompt
+				// files that are resolved asynchronously, hence we need to wait for
+				// the entire prompt instruction tree to be processed
+				const instructionsStarted = performance.now();
+				await promptInstructions.allSettled();
+				// allow-any-unicode-next-line
+				this.logService.trace(`[⏱] instructions tree resolved in ${performance.now() - instructionsStarted}ms`);
+			}
 
 			let attachedContext = this.inputPart.getAttachedAndImplicitContext(this.viewModel.sessionId);
 			let workingSet: URI[] | undefined;
-			if (this.location === ChatAgentLocation.EditingSession) {
-				const currentEditingSession = this.chatEditingService.currentEditingSessionObs.get();
+			if (this.viewOptions.enableWorkingSet !== undefined) {
+				const currentEditingSession = this._editingSession;
 
 				const unconfirmedSuggestions = new ResourceSet();
 				const uniqueWorkingSetEntries = new ResourceSet(); // NOTE: this is used for bookkeeping so the UI can avoid rendering references in the UI that are already shown in the working set
 				const editingSessionAttachedContext: IChatRequestVariableEntry[] = [];
 				// Pick up everything that the user sees is part of the working set.
 				// This should never exceed the maximum file entries limit above.
-				for (const v of this.inputPart.chatEditWorkingSetFiles) {
+				for (const { uri, isMarkedReadonly } of this.inputPart.chatEditWorkingSetFiles) {
 					// Skip over any suggested files that haven't been confirmed yet in the working set
-					if (currentEditingSession?.workingSet.get(v)?.state === WorkingSetEntryState.Suggested) {
-						unconfirmedSuggestions.add(v);
+					if (currentEditingSession?.workingSet.get(uri)?.state === WorkingSetEntryState.Suggested) {
+						unconfirmedSuggestions.add(uri);
 					} else {
-						uniqueWorkingSetEntries.add(v);
-						editingSessionAttachedContext.push(this.attachmentModel.asVariableEntry(v));
+						uniqueWorkingSetEntries.add(uri);
+						editingSessionAttachedContext.push(this.attachmentModel.asVariableEntry(uri, undefined, isMarkedReadonly));
 					}
 				}
 				let maximumFileEntries = this.chatEditingService.editingSessionFileLimit - editingSessionAttachedContext.length;
@@ -1032,6 +1074,13 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					}
 				}
 
+				// add prompt instruction references to the attached context, if enabled
+				const promptInstructionUris = new ResourceSet(promptInstructions.chatAttachments.map((v) => v.value) as URI[]);
+				if (instructionsEnabled) {
+					editingSessionAttachedContext
+						.push(...promptInstructions.chatAttachments);
+				}
+
 				for (const file of uniqueWorkingSetEntries) {
 					// Make sure that any files that we sent are part of the working set
 					// but do not permanently add file variables from previous requests to the working set
@@ -1045,7 +1094,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					for (const variable of request.variableData.variables) {
 						if (URI.isUri(variable.value) && variable.isFile && maximumFileEntries > 0) {
 							const uri = variable.value;
-							if (!uniqueWorkingSetEntries.has(uri)) {
+							if (!uniqueWorkingSetEntries.has(uri) && !promptInstructionUris.has(uri)) {
 								editingSessionAttachedContext.push(variable);
 								uniqueWorkingSetEntries.add(variable.value);
 								maximumFileEntries -= 1;
@@ -1070,6 +1119,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				currentEditingSession?.remove(WorkingSetEntryRemovalReason.User, ...unconfirmedSuggestions);
 			}
 
+			this.chatService.cancelCurrentRequestForSession(this.viewModel.sessionId);
+
 			const result = await this.chatService.sendRequest(this.viewModel.sessionId, input, {
 				userSelectedModelId: this.inputPart.currentLanguageModel,
 				location: this.location,
@@ -1078,6 +1129,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				attachedContext,
 				workingSet,
 				noCommandDetection: options?.noCommandDetection,
+				hasInstructionAttachments: this.inputPart.hasInstructionAttachments,
 			});
 
 			if (result) {
@@ -1095,6 +1147,13 @@ export class ChatWidget extends Disposable implements IChatWidget {
 						}
 					}
 				});
+
+				const RESPONSE_TIMEOUT = 20000;
+				setTimeout(() => {
+					// Stop the signal if the promise is still unresolved
+					this.chatAccessibilityService.acceptResponse(undefined, requestId, options?.isVoiceInput);
+				}, RESPONSE_TIMEOUT);
+
 				return result.responseCreatedPromise;
 			}
 		}
