@@ -24,11 +24,6 @@ export interface IChatEditingService {
 
 	_serviceBrand: undefined;
 
-	/**
-	 * emitted when a session is created, changed or disposed
-	 */
-	readonly onDidChangeEditingSession: Event<void>;
-
 	readonly currentEditingSessionObs: IObservable<IChatEditingSession | null>;
 
 	readonly currentEditingSession: IChatEditingSession | null;
@@ -38,9 +33,21 @@ export interface IChatEditingService {
 
 	startOrContinueEditingSession(chatSessionId: string): Promise<IChatEditingSession>;
 	getOrRestoreEditingSession(): Promise<IChatEditingSession | null>;
+
+
 	hasRelatedFilesProviders(): boolean;
 	registerRelatedFilesProvider(handle: number, provider: IChatRelatedFilesProvider): IDisposable;
 	getRelatedFiles(chatSessionId: string, prompt: string, token: CancellationToken): Promise<{ group: string; files: IChatRelatedFile[] }[] | undefined>;
+
+	/**
+	 * All editing sessions, sorted by recency, e.g the last created session comes first.
+	 */
+	readonly editingSessionsObs: IObservable<readonly IChatEditingSession[]>;
+
+	/**
+	 * Creates a new short lived editing session
+	 */
+	createAdhocEditingSession(chatSessionId: string): Promise<IChatEditingSession & IDisposable>;
 }
 
 export interface IChatRequestDraft {
@@ -62,19 +69,25 @@ export interface IChatRelatedFilesProvider {
 	provideRelatedFiles(chatRequest: IChatRequestDraft, token: CancellationToken): Promise<IChatRelatedFile[] | undefined>;
 }
 
-export interface WorkingSetDisplayMetadata { state: WorkingSetEntryState; description?: string }
+export interface WorkingSetDisplayMetadata {
+	state: WorkingSetEntryState;
+	description?: string;
+	isMarkedReadonly?: boolean;
+}
 
 export interface IChatEditingSession {
+	readonly isGlobalEditingSession: boolean;
 	readonly chatSessionId: string;
 	readonly onDidChange: Event<ChatEditingSessionChangeType>;
 	readonly onDidDispose: Event<void>;
 	readonly state: IObservable<ChatEditingSessionState>;
 	readonly entries: IObservable<readonly IModifiedFileEntry[]>;
 	readonly workingSet: ResourceMap<WorkingSetDisplayMetadata>;
-	readonly isVisible: boolean;
+	readonly isToolsAgentSession: boolean;
 	addFileToWorkingSet(uri: URI, description?: string, kind?: WorkingSetEntryState.Transient | WorkingSetEntryState.Suggested): void;
 	show(): Promise<void>;
 	remove(reason: WorkingSetEntryRemovalReason, ...uris: URI[]): void;
+	markIsReadonly(uri: URI, isReadonly?: boolean): void;
 	accept(...uris: URI[]): Promise<void>;
 	reject(...uris: URI[]): Promise<void>;
 	getEntry(uri: URI): IModifiedFileEntry | undefined;
@@ -126,6 +139,10 @@ export interface IModifiedFileEntry {
 	readonly lastModifyingRequestId: string;
 	accept(transaction: ITransaction | undefined): Promise<void>;
 	reject(transaction: ITransaction | undefined): Promise<void>;
+
+	reviewMode: IObservable<boolean>;
+	autoAcceptController: IObservable<{ total: number; remaining: number; cancel(): void } | undefined>;
+	enableReviewModeUntilSettled(): void;
 }
 
 export interface IChatEditingSessionStream {
@@ -142,6 +159,8 @@ export const enum ChatEditingSessionState {
 export const CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME = 'chat-editing-multi-diff-source';
 
 export const chatEditingWidgetFileStateContextKey = new RawContextKey<WorkingSetEntryState>('chatEditingWidgetFileState', undefined, localize('chatEditingWidgetFileState', "The current state of the file in the chat editing widget"));
+export const chatEditingWidgetFileReadonlyContextKey = new RawContextKey<boolean>('chatEditingWidgetFileReadonly', undefined, localize('chatEditingWidgetFileReadonly', "Whether the file has been marked as read-only in the chat editing widget"));
+export const chatEditingAgentSupportsReadonlyReferencesContextKey = new RawContextKey<boolean>('chatEditingAgentSupportsReadonlyReferences', undefined, localize('chatEditingAgentSupportsReadonlyReferences', "Whether the chat editing agent supports readonly references (temporary)"));
 export const decidedChatEditingResourceContextKey = new RawContextKey<string[]>('decidedChatEditingResource', []);
 export const chatEditingResourceContextKey = new RawContextKey<string | undefined>('chatEditingResource', undefined);
 export const inChatEditingSessionContextKey = new RawContextKey<boolean | undefined>('inChatEditingSession', undefined);
