@@ -16,13 +16,13 @@ import { Range } from '../../../../editor/common/core/range.js';
 import { ScrollType } from '../../../../editor/common/editorCommon.js';
 import { IOptions, ZoneWidget } from '../../../../editor/contrib/zoneWidget/browser/zoneWidget.js';
 import { localize } from '../../../../nls.js';
-import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
 import { IContextKey, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
 import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
 import { ILogService } from '../../../../platform/log/common/log.js';
+import { IChatWidgetViewOptions } from '../../chat/browser/chat.js';
 import { IChatWidgetLocationOptions } from '../../chat/browser/chatWidget.js';
 import { isResponseVM } from '../../chat/common/chatViewModel.js';
-import { ACTION_REGENERATE_RESPONSE, ACTION_REPORT_ISSUE, ACTION_TOGGLE_DIFF, CTX_INLINE_CHAT_OUTER_CURSOR_POSITION, EditMode, InlineChatConfigKeys, MENU_INLINE_CHAT_WIDGET_SECONDARY, MENU_INLINE_CHAT_WIDGET_STATUS } from '../common/inlineChat.js';
+import { ACTION_REGENERATE_RESPONSE, ACTION_REPORT_ISSUE, ACTION_TOGGLE_DIFF, CTX_INLINE_CHAT_OUTER_CURSOR_POSITION, MENU_INLINE_CHAT_WIDGET_SECONDARY, MENU_INLINE_CHAT_WIDGET_STATUS } from '../common/inlineChat.js';
 import { EditorBasedInlineChatWidget } from './inlineChatWidget.js';
 
 export class InlineChatZoneWidget extends ZoneWidget {
@@ -48,11 +48,11 @@ export class InlineChatZoneWidget extends ZoneWidget {
 
 	constructor(
 		location: IChatWidgetLocationOptions,
+		options: IChatWidgetViewOptions | undefined,
 		editor: ICodeEditor,
 		@IInstantiationService private readonly _instaService: IInstantiationService,
 		@ILogService private _logService: ILogService,
 		@IContextKeyService contextKeyService: IContextKeyService,
-		@IConfigurationService configurationService: IConfigurationService,
 	) {
 		super(editor, InlineChatZoneWidget._options);
 
@@ -82,15 +82,15 @@ export class InlineChatZoneWidget extends ZoneWidget {
 				menus: {
 					telemetrySource: 'interactiveEditorWidget-toolbar',
 				},
+				...options,
 				rendererOptions: {
 					renderTextEditsAsSummary: (uri) => {
-						// render edits as summary only when using Live mode and when
-						// dealing with the current file in the editor
-						return isEqual(uri, editor.getModel()?.uri)
-							&& configurationService.getValue<EditMode>(InlineChatConfigKeys.Mode) === EditMode.Live;
+						// render when dealing with the current file in the editor
+						return isEqual(uri, editor.getModel()?.uri);
 					},
 					renderDetectedCommandsWithRequest: true,
-				}
+					...options?.rendererOptions
+				},
 			}
 		});
 		this._disposables.add(this.widget);
