@@ -17,9 +17,11 @@ import { ITreeContextMenuEvent, ITreeFilter, ITreeNode, ITreeRenderer, ITreeSort
 import { Action, ActionRunner, IAction, Separator } from '../../../../base/common/actions.js';
 import { mapFindFirst } from '../../../../base/common/arraysFind.js';
 import { RunOnceScheduler, disposableTimeout } from '../../../../base/common/async.js';
+import { groupBy } from '../../../../base/common/collections.js';
 import { Color, RGBA } from '../../../../base/common/color.js';
 import { Emitter, Event } from '../../../../base/common/event.js';
 import { FuzzyScore } from '../../../../base/common/filters.js';
+import { Iterable } from '../../../../base/common/iterator.js';
 import { KeyCode } from '../../../../base/common/keyCodes.js';
 import { Disposable, DisposableStore, MutableDisposable } from '../../../../base/common/lifecycle.js';
 import { autorun, observableFromEvent } from '../../../../base/common/observable.js';
@@ -65,7 +67,7 @@ import { ITestProfileService, canUseProfileWithTest } from '../common/testProfil
 import { LiveTestResult, TestResultItemChangeReason } from '../common/testResult.js';
 import { ITestResultService } from '../common/testResultService.js';
 import { IMainThreadTestCollection, ITestService, testCollectionIsEmpty } from '../common/testService.js';
-import { ITestRunProfile, InternalTestItem, TestControllerCapability, TestItemExpandState, TestResultState, TestRunProfileBitset, testResultStateToContextValues } from '../common/testTypes.js';
+import { ITestRunProfile, InternalTestItem, TestControllerCapability, TestItemExpandState, TestResultState, TestRunProfileBitset, testProfileBitset, testResultStateToContextValues } from '../common/testTypes.js';
 import { TestingContextKeys } from '../common/testingContextKeys.js';
 import { ITestingContinuousRunService } from '../common/testingContinuousRunService.js';
 import { ITestingPeekOpener } from '../common/testingPeekOpener.js';
@@ -118,6 +120,7 @@ export class TestingExplorerView extends ViewPane {
 		@ITestProfileService private readonly testProfileService: ITestProfileService,
 		@ICommandService private readonly commandService: ICommandService,
 		@IMenuService private readonly menuService: IMenuService,
+		@ITestingContinuousRunService private readonly crService: ITestingContinuousRunService,
 	) {
 		super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService, hoverService);
 
@@ -128,6 +131,10 @@ export class TestingExplorerView extends ViewPane {
 			}
 		}));
 
+		this._register(Event.any(crService.onDidChange, testProfileService.onDidChange)(() => {
+			this.updateActions();
+		}));
+
 		this._register(testService.collection.onBusyProvidersChange(busy => {
 			this.updateDiscoveryProgress(busy);
 		}));
@@ -326,6 +333,9 @@ export class TestingExplorerView extends ViewPane {
 				return this.getRunGroupDropdown(TestRunProfileBitset.Run, action, options);
 			case TestCommandId.DebugSelectedAction:
 				return this.getRunGroupDropdown(TestRunProfileBitset.Debug, action, options);
+			case TestCommandId.StartContinousRun:
+			case TestCommandId.StopContinousRun:
+				return this.getContinuousRunDropdown(action, options);
 			default:
 				return super.getActionViewItem(action, options);
 		}
@@ -336,6 +346,7 @@ export class TestingExplorerView extends ViewPane {
 		const profileActions: IAction[] = [];
 
 		let participatingGroups = 0;
+		let participatingProfiles = 0;
 		let hasConfigurable = false;
 		const defaults = this.testProfileService.getGroupDefaultProfiles(group);
 		for (const { profiles, controller } of this.testProfileService.all()) {
@@ -353,6 +364,7 @@ export class TestingExplorerView extends ViewPane {
 				}
 
 				hasConfigurable = hasConfigurable || profile.hasConfigurationHandler;
+				participatingProfiles++;
 				profileActions.push(new Action(
 					`${controller.id}.${profile.profileId}`,
 					defaults.includes(profile) ? localize('defaultTestProfile', '{0} (Default)', profile.label) : profile.label,
@@ -392,7 +404,7 @@ export class TestingExplorerView extends ViewPane {
 		const menuActions = getFlatContextMenuActions(menu);
 
 		const postActions: IAction[] = [];
-		if (profileActions.length > 1) {
+		if (participatingProfiles > 1) {
 			postActions.push(new Action(
 				'selectDefaultTestConfigurations',
 				localize('selectDefaultConfigs', 'Select Default Profile'),
@@ -413,9 +425,12 @@ export class TestingExplorerView extends ViewPane {
 		}
 
 		// show menu actions if there are any otherwise don't
-		return menuActions.length > 0
-			? Separator.join(profileActions, menuActions, postActions)
-			: Separator.join(profileActions, postActions);
+		return {
+			numberOfProfiles: participatingProfiles,
+			actions: menuActions.length > 0
+				? Separator.join(profileActions, menuActions, postActions)
+				: Separator.join(profileActions, postActions),
+		};
 	}
 
 	/**
@@ -428,7 +443,7 @@ export class TestingExplorerView extends ViewPane {
 
 	private getRunGroupDropdown(group: TestRunProfileBitset, defaultAction: IAction, options: IActionViewItemOptions) {
 		const dropdownActions = this.getTestConfigGroupActions(group);
-		if (dropdownActions.length < 2) {
+		if (dropdownActions.numberOfProfiles < 2) {
 			return super.getActionViewItem(defaultAction, options);
 		}
 
@@ -440,11 +455,74 @@ export class TestingExplorerView extends ViewPane {
 				: icons.testingDebugAllIcon,
 		}, undefined, undefined, undefined, undefined);
 
-		const dropdownAction = new Action('selectRunConfig', 'Select Configuration...', 'codicon-chevron-down', true);
+		return this.instantiationService.createInstance(
+			DropdownWithPrimaryActionViewItem,
+			primaryAction, this.getDropdownAction(), dropdownActions.actions,
+			'',
+			options
+		);
+	}
+
+	private getDropdownAction() {
+		return new Action('selectRunConfig', localize('testingSelectConfig', 'Select Configuration...'), 'codicon-chevron-down', true);
+	}
+
+	private getContinuousRunDropdown(defaultAction: IAction, options: IActionViewItemOptions) {
+		const allProfiles = [...Iterable.flatMap(this.testProfileService.all(), (cr): Iterable<ITestRunProfile> => {
+			if (this.testService.collection.getNodeById(cr.controller.id)?.children.size) {
+				return Iterable.filter(cr.profiles, p => p.supportsContinuousRun);
+			}
+			return Iterable.empty();
+		})];
+
+		if (allProfiles.length <= 1) {
+			return super.getActionViewItem(defaultAction, options);
+		}
+
+		const primaryAction = this.instantiationService.createInstance(MenuItemAction, {
+			id: defaultAction.id,
+			title: defaultAction.label,
+			icon: defaultAction.id === TestCommandId.StartContinousRun ? icons.testingTurnContinuousRunOn : icons.testingTurnContinuousRunOff,
+		}, undefined, undefined, undefined, undefined);
+
+		const dropdownActions: IAction[] = [];
+		const groups = groupBy(allProfiles, p => p.group);
+		const crService = this.crService;
+		for (const group of [TestRunProfileBitset.Run, TestRunProfileBitset.Debug, TestRunProfileBitset.Coverage] as const) {
+			const profiles = groups[group];
+			if (!profiles) {
+				continue;
+			}
+
+			if (Object.keys(groups).length > 1) {
+				dropdownActions.push({
+					id: `${group}.label`,
+					label: testProfileBitset[group],
+					enabled: false,
+					class: undefined,
+					tooltip: testProfileBitset[group],
+					run: () => { },
+				});
+			}
+
+			for (const profile of profiles) {
+				dropdownActions.push({
+					id: `${group}.${profile.profileId}`,
+					label: profile.label,
+					enabled: true,
+					class: undefined,
+					tooltip: profile.label,
+					checked: crService.isEnabledForProfile(profile),
+					run: () => crService.isEnabledForProfile(profile)
+						? crService.stopProfile(profile)
+						: crService.start([profile]),
+				});
+			}
+		}
 
 		return this.instantiationService.createInstance(
 			DropdownWithPrimaryActionViewItem,
-			primaryAction, dropdownAction, dropdownActions,
+			primaryAction, this.getDropdownAction(), dropdownActions,
 			'',
 			options
 		);
@@ -727,7 +805,6 @@ class TestingExplorerViewModel extends Disposable {
 				accessibilityProvider: instantiationService.createInstance(ListAccessibilityProvider),
 				filter: this.filter,
 				findWidgetEnabled: false,
-				openOnSingleClick: false,
 			}) as TestingObjectTree<FuzzyScore>;
 
 
@@ -786,8 +863,19 @@ class TestingExplorerViewModel extends Disposable {
 		}));
 
 		this._register(this.tree.onDidOpen(e => {
-			if (e.element instanceof TestItemTreeElement && !e.element.children.size && e.element.test.item.uri) {
-				commandService.executeCommand('vscode.revealTest', e.element.test.item.extId);
+			if (!(e.element instanceof TestItemTreeElement)) {
+				return;
+			}
+
+			filterState.didSelectTestInExplorer(e.element.test.item.extId);
+
+			if (!e.element.children.size && e.element.test.item.uri) {
+				if (!this.tryPeekError(e.element)) {
+					commandService.executeCommand('vscode.revealTest', e.element.test.item.extId, {
+						openToSide: e.sideBySide,
+						preserveFocus: true,
+					});
+				}
 			}
 		}));
 
@@ -816,20 +904,6 @@ class TestingExplorerViewModel extends Disposable {
 			}
 		}));
 
-		this._register(this.tree.onDidChangeSelection(evt => {
-			if (dom.isMouseEvent(evt.browserEvent) && (evt.browserEvent.altKey || evt.browserEvent.shiftKey)) {
-				return; // don't focus when alt-clicking to multi select
-			}
-
-			const selected = evt.elements[0];
-			if (selected && evt.browserEvent && selected instanceof TestItemTreeElement) {
-				filterState.didSelectTestInExplorer(selected.test.item.extId);
-				if (selected.children.size === 0 && selected.test.expand === TestItemExpandState.NotExpandable) {
-					this.tryPeekError(selected);
-				}
-			}
-		}));
-
 		let followRunningTests = getTestingConfiguration(configurationService, TestingConfigKeys.FollowRunningTest);
 		this._register(configurationService.onDidChangeConfiguration(e => {
 			if (e.affectsConfiguration(TestingConfigKeys.FollowRunningTest)) {
@@ -1457,8 +1531,8 @@ class TestItemRenderer extends Disposable
 	/**
 	 * @inheritdoc
 	 */
-	public renderTemplate(container: HTMLElement): ITestElementTemplateData {
-		const wrapper = dom.append(container, dom.$('.test-item'));
+	public renderTemplate(wrapper: HTMLElement): ITestElementTemplateData {
+		wrapper.classList.add('testing-stdtree-container');
 
 		const icon = dom.append(wrapper, dom.$('.computed-state'));
 		const label = dom.append(wrapper, dom.$('.label'));
