@@ -67,7 +67,8 @@ export type BuiltinGettingStartedStep = {
 	media:
 	| { type: 'image'; path: string | { hc: string; hcLight?: string; light: string; dark: string }; altText: string }
 	| { type: 'svg'; path: string; altText: string }
-	| { type: 'markdown'; path: string };
+	| { type: 'markdown'; path: string }
+	| { type: 'video'; path: string | { hc: string; hcLight?: string; light: string; dark: string }; poster?: string | { hc: string; hcLight?: string; light: string; dark: string }; altText: string };
 };
 
 export type BuiltinGettingStartedCategory = {
@@ -232,7 +233,6 @@ function createCopilotSetupStep(id: string, button: string, when: string, includ
 	};
 }
 
-
 export const walkthroughs: GettingStartedWalkthroughContent = [
 	{
 		id: 'Setup',
