@@ -124,7 +124,7 @@ import './services/editor/common/customEditorLabelService.js';
 import { InstantiationType, registerSingleton } from '../platform/instantiation/common/extensions.js';
 import { ExtensionGalleryService } from '../platform/extensionManagement/common/extensionGalleryService.js';
 import { GlobalExtensionEnablementService } from '../platform/extensionManagement/common/extensionEnablementService.js';
-import { IExtensionGalleryService, IGlobalExtensionEnablementService } from '../platform/extensionManagement/common/extensionManagement.js';
+import { IAllowedExtensionsService, IExtensionGalleryService, IGlobalExtensionEnablementService } from '../platform/extensionManagement/common/extensionManagement.js';
 import { ContextViewService } from '../platform/contextview/browser/contextViewService.js';
 import { IContextViewService } from '../platform/contextview/browser/contextView.js';
 import { IListService, ListService } from '../platform/list/browser/listService.js';
@@ -148,6 +148,7 @@ import { IUserDataSyncLogService } from '../platform/userDataSync/common/userDat
 import { UserDataSyncLogService } from '../platform/userDataSync/common/userDataSyncLog.js';
 
 registerSingleton(IUserDataSyncLogService, UserDataSyncLogService, InstantiationType.Delayed);
+registerSingleton(IAllowedExtensionsService, AllowedExtensionsService, InstantiationType.Delayed);
 registerSingleton(IIgnoredExtensionsManagementService, IgnoredExtensionsManagementService, InstantiationType.Delayed);
 registerSingleton(IGlobalExtensionEnablementService, GlobalExtensionEnablementService, InstantiationType.Delayed);
 registerSingleton(IExtensionStorageService, ExtensionStorageService, InstantiationType.Delayed);
@@ -245,9 +246,6 @@ import './contrib/mergeEditor/browser/mergeEditor.contribution.js';
 // Multi Diff Editor
 import './contrib/multiDiffEditor/browser/multiDiffEditor.contribution.js';
 
-// Mapped Edits
-import './contrib/mappedEdits/common/mappedEdits.contribution.js';
-
 // Commands
 import './contrib/commands/common/commands.contribution.js';
 
@@ -271,7 +269,6 @@ import './contrib/extensions/browser/extensions.contribution.js';
 import './contrib/extensions/browser/extensionsViewlet.js';
 
 // Output View
-import './contrib/output/common/outputChannelModelService.js';
 import './contrib/output/browser/output.contribution.js';
 import './contrib/output/browser/outputView.js';
 
@@ -398,4 +395,9 @@ import './contrib/scrollLocking/browser/scrollLocking.contribution.js';
 // Inline Completions
 import './contrib/inlineCompletions/browser/inlineCompletions.contribution.js';
 
+// Drop or paste into
+import './contrib/dropOrPasteInto/browser/dropOrPasteInto.contribution.js';
+import { AllowedExtensionsService } from '../platform/extensionManagement/common/allowedExtensionsService.js';
+
+
 //#endregion
